# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


class PgoTaskReviewsToCreateStatus(Enum):
    """An enumeration representing different categories.

    :cvar APPROVED: "approved"
    :vartype APPROVED: str
    :cvar DECLINED: "declined"
    :vartype DECLINED: str
    """

    APPROVED = "approved"
    DECLINED = "declined"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PgoTaskReviewsToCreateStatus._member_map_.values())
        )


@JsonMap({})
class PgoTaskReviewsToCreate(BaseModel):
    """PgoTaskReviewsToCreate

    :param status: Review status.
    :type status: PgoTaskReviewsToCreateStatus
    :param reason: Additional comments with the reivew., defaults to None
    :type reason: str, optional
    :param ids: ids
    :type ids: List[str]
    """

    def __init__(
        self, status: PgoTaskReviewsToCreateStatus, ids: List[str], reason: str = None
    ):
        self.status = self._enum_matching(
            status, PgoTaskReviewsToCreateStatus.list(), "status"
        )
        if reason is not None:
            self.reason = reason
        self.ids = ids



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.upload_employee_compliance_document_file_container import (
    UploadEmployeeComplianceDocumentFileContainer,
)
from ..models.upload_employee_compliance_document_container import (
    UploadEmployeeComplianceDocumentContainer,
)
from ..models.request_custom_verification_letter_container import (
    RequestCustomVerificationLetterContainer,
)
from ..models.hr_verification_letters_and_documents_list_container import (
    HrVerificationLettersAndDocumentsListContainer,
)
from ..models.get_employee_compliance_documents_container import (
    GetEmployeeComplianceDocumentsContainer,
)
from ..models.get_employee_compliance_document_template_download_link_container import (
    GetEmployeeComplianceDocumentTemplateDownloadLinkContainer,
)
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_created import GenericResultCreated
from ..models.employee_tax_documents_list_container import (
    EmployeeTaxDocumentsListContainer,
)
from ..models.employee_payslips_list_container import EmployeePayslipsListContainer
from ..models.employee_contract_signature_to_create_container import (
    EmployeeContractSignatureToCreateContainer,
)
from ..models.employee_agreement_download_container import (
    EmployeeAgreementDownloadContainer,
)
from ..models.bank_account_to_add_container import BankAccountToAddContainer
from ..models.bank_account_guide_container import BankAccountGuideContainer
from ..models.bank_account_added_container import BankAccountAddedContainer
from ..models.additional_eor_info_container import AdditionalEorInfoContainer


class PartnerManagedService(BaseService):

    @cast_models
    def add_employee_additional_information(
        self,
        request_body: AdditionalEorInfoContainer,
        employee_id: str,
        contract_id: str,
    ) -> GenericResultUpdated:
        """Add additional information for an EOR employee.

        :param request_body: The request body.
        :type request_body: AdditionalEorInfoContainer
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(AdditionalEorInfoContainer).validate(request_body)
        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/additional-information",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def sign_employee_contract(
        self,
        request_body: EmployeeContractSignatureToCreateContainer,
        employee_id: str,
        contract_id: str,
    ) -> GenericResultCreated:
        """Sign a contract as a employee.

        :param request_body: The request body.
        :type request_body: EmployeeContractSignatureToCreateContainer
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(EmployeeContractSignatureToCreateContainer).validate(request_body)
        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/signatures",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def request_custom_verification_letter(
        self,
        request_body: RequestCustomVerificationLetterContainer,
        employee_id: str,
        contract_id: str,
    ) -> GenericResultCreated:
        """Request employment verification letters, visa support, bank verification and more.

        :param request_body: The request body.
        :type request_body: RequestCustomVerificationLetterContainer
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(RequestCustomVerificationLetterContainer).validate(request_body)
        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/custom-verification-letter",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def get_hr_verification_letters_and_documents(
        self, employee_id: str, contract_id: str
    ) -> HrVerificationLettersAndDocumentsListContainer:
        """List all HR verification letters and documents available.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: HrVerificationLettersAndDocumentsListContainer
        """

        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/hr-documents",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return HrVerificationLettersAndDocumentsListContainer._unmap(response)

    @cast_models
    def download_hr_verification_letters_and_documents(
        self, employee_id: str, contract_id: str, document_id: float
    ) -> EmployeeAgreementDownloadContainer:
        """Retrieve URL to download HR verification letters and documents.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        :param document_id: Unique identifier for a compliance document in Deel.
        :type document_id: float
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EmployeeAgreementDownloadContainer
        """

        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)
        Validator(float).validate(document_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/hr-documents/{{document_id}}/download",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .add_path("document_id", document_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeAgreementDownloadContainer._unmap(response)

    @cast_models
    def get_offer_letter_preview(self, employee_id: str, contract_id: str) -> str:
        """Retrieve an EOR job offer letter in HTML. This endpoint does not support IC and Global Payroll contract types.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: str
        """

        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/offer-letter",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def get_employee_agreement_preview(self, employee_id: str, contract_id: str) -> str:
        """Retrieve an EOR Employee Agreement content in HTML.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: str
        """

        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/employee-agreement",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def get_employee_agreement_download_link(
        self, employee_id: str, contract_id: str
    ) -> EmployeeAgreementDownloadContainer:
        """Get link to download the employee agreement PDF.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EmployeeAgreementDownloadContainer
        """

        Validator(str).validate(employee_id)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/contracts/{{contract_id}}/employee-agreement/download",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeAgreementDownloadContainer._unmap(response)

    @cast_models
    def get_bank_account_guide(self, employee_id: str) -> BankAccountGuideContainer:
        """Retrieve bank account form guide for an EOR employee. This data can be used to add a new bank account for an employee.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountGuideContainer
        """

        Validator(str).validate(employee_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/banks/guide",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return BankAccountGuideContainer._unmap(response)

    @cast_models
    def add_bank_account(
        self, request_body: BankAccountToAddContainer, employee_id: str
    ) -> BankAccountAddedContainer:
        """Add bank account for an EOR employee.

        :param request_body: The request body.
        :type request_body: BankAccountToAddContainer
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountAddedContainer
        """

        Validator(BankAccountToAddContainer).validate(request_body)
        Validator(str).validate(employee_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/banks",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return BankAccountAddedContainer._unmap(response)

    @cast_models
    def patch_bank_account(
        self, request_body: BankAccountToAddContainer, employee_id: str, bank_id: str
    ) -> BankAccountAddedContainer:
        """Modify bank account for an EOR employee.

        :param request_body: The request body.
        :type request_body: BankAccountToAddContainer
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param bank_id: Bank Account id.
        :type bank_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountAddedContainer
        """

        Validator(BankAccountToAddContainer).validate(request_body)
        Validator(str).validate(employee_id)
        Validator(str).validate(bank_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/banks/{{bank_id}}",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("bank_id", bank_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return BankAccountAddedContainer._unmap(response)

    @cast_models
    def get_employee_payslips(self, employee_id: str) -> EmployeePayslipsListContainer:
        """Get list of payslips for an EOR employee.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EmployeePayslipsListContainer
        """

        Validator(str).validate(employee_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/payslips",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeePayslipsListContainer._unmap(response)

    @cast_models
    def get_employee_compliance_documents(
        self, employee_id: str
    ) -> GetEmployeeComplianceDocumentsContainer:
        """Get a list of employee compliance documents.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetEmployeeComplianceDocumentsContainer
        """

        Validator(str).validate(employee_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/compliance-documents",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetEmployeeComplianceDocumentsContainer._unmap(response)

    @cast_models
    def upload_employee_compliance_document(
        self, request_body: dict, employee_id: str, document_id: float
    ) -> UploadEmployeeComplianceDocumentContainer:
        """Upload an employee compliance document.

        :param request_body: The request body.
        :type request_body: dict
        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param document_id: Unique identifier for a compliance document in Deel.
        :type document_id: float
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: UploadEmployeeComplianceDocumentContainer
        """

        Validator(dict).validate(request_body)
        Validator(str).validate(employee_id)
        Validator(float).validate(document_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/compliance-documents/{{document_id}}",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("document_id", document_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return UploadEmployeeComplianceDocumentContainer._unmap(response)

    @cast_models
    def get_employee_compliance_document_template(
        self, employee_id: str, document_id: float
    ) -> GetEmployeeComplianceDocumentTemplateDownloadLinkContainer:
        """Get the download link for an employee compliance document template, if it exists.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        :param document_id: Unique identifier for a compliance document in Deel.
        :type document_id: float
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetEmployeeComplianceDocumentTemplateDownloadLinkContainer
        """

        Validator(str).validate(employee_id)
        Validator(float).validate(document_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/compliance-documents/{{document_id}}/templates/download",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .add_path("document_id", document_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetEmployeeComplianceDocumentTemplateDownloadLinkContainer._unmap(
            response
        )

    @cast_models
    def get_employee_tax_documents(
        self, employee_id: str
    ) -> EmployeeTaxDocumentsListContainer:
        """Get list of tax documents for an employee.

        :param employee_id: Unique identifier for an employee in Deel.
        :type employee_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EmployeeTaxDocumentsListContainer
        """

        Validator(str).validate(employee_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/partner-managed/employees/{{employee_id}}/tax-documents",
                self.get_default_headers(),
            )
            .add_path("employee_id", employee_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeTaxDocumentsListContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


class ContractTemplateIdGuard(OneOfBaseModel):
    class_list = {"float": float, "str": str}


ContractTemplateId = Union[float, str]


@JsonMap({})
class ContractSignatureToCreate(BaseModel):
    """ContractSignatureToCreate

    :param client_signature: Signature of client.
    :type client_signature: str
    :param contract_template_id: Please use null to use default Deel contract template; otherwise you can use the template you created and customized. If template ID is not in payload, template stays unchanged., defaults to None
    :type contract_template_id: ContractTemplateId, optional
    """

    def __init__(
        self, client_signature: str, contract_template_id: ContractTemplateId = None
    ):
        self.client_signature = client_signature
        if contract_template_id is not None:
            self.contract_template_id = ContractTemplateIdGuard.return_one_of(
                contract_template_id
            )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class HrisTeamInformation(BaseModel):
    """HrisTeamInformation

    :param team_id: team_id
    :type team_id: float
    :param legal_entity_id: legal_entity_id
    :type legal_entity_id: float
    """

    def __init__(self, team_id: float, legal_entity_id: float):
        self.team_id = team_id
        self.legal_entity_id = legal_entity_id



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class UploadEmployeeComplianceDocumentFileContainer(BaseModel):
    """UploadEmployeeComplianceDocumentFileContainer

    :param file: The employee compliance document to upload.
    :type file: str
    """

    def __init__(self, file: str):
        self.file = file



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"street_address": "streetAddress", "postal_code": "postalCode"})
class Address(BaseModel):
    """Address

    :param street_address: Street and number., defaults to None
    :type street_address: str, optional
    :param locality: Locality name., defaults to None
    :type locality: str, optional
    :param region: Region code., defaults to None
    :type region: str, optional
    :param postal_code: Postal code., defaults to None
    :type postal_code: str, optional
    :param country: Country code., defaults to None
    :type country: str, optional
    """

    def __init__(
        self,
        street_address: str = None,
        locality: str = None,
        region: str = None,
        postal_code: str = None,
        country: str = None,
    ):
        if street_address is not None:
            self.street_address = street_address
        if locality is not None:
            self.locality = locality
        if region is not None:
            self.region = region
        if postal_code is not None:
            self.postal_code = postal_code
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_status_enum import InvoiceStatusEnum


@JsonMap({"id_": "id"})
class Invoice(BaseModel):
    """Invoice

    :param id_: Unique identifier of the invoice.
    :type id_: str
    :param status: Current status of the invoice.
    :type status: InvoiceStatusEnum
    :param currency: Currency code.
    :type currency: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param total: Total invoice amount including Fee and VAT.
    :type total: str
    :param label: Label of the invoice.
    :type label: str
    :param paid_at: Long date-time format following ISO-8601
    :type paid_at: str
    :param vat_total: Amount of VAT.
    :type vat_total: str
    :param vat_percentage: VAT percentage.
    :type vat_percentage: str
    :param is_overdue: Indicates if the invoice is overdue.
    :type is_overdue: bool
    :param issued_at: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31.
    :type issued_at: str
    :param vat_id: VAT identification number.
    :type vat_id: str
    :param due_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31.
    :type due_date: str
    :param contract_id: Unique identifier of the contract.
    :type contract_id: str
    """

    def __init__(
        self,
        id_: str,
        status: InvoiceStatusEnum,
        currency: str,
        created_at: str,
        total: str,
        label: str,
        paid_at: str,
        vat_total: str,
        vat_percentage: str,
        is_overdue: bool,
        issued_at: str,
        vat_id: str,
        due_date: str,
        contract_id: str,
    ):
        self.id_ = id_
        self.status = self._enum_matching(status, InvoiceStatusEnum.list(), "status")
        self.currency = self._pattern_matching(currency, "^[A-Z]{3}$", "currency")
        self.created_at = created_at
        self.total = total
        self.label = label
        self.paid_at = paid_at
        self.vat_total = vat_total
        self.vat_percentage = vat_percentage
        self.is_overdue = is_overdue
        self.issued_at = self._pattern_matching(
            issued_at, "^\d{4}-\d{2}-\d{2}$", "issued_at"
        )
        self.vat_id = vat_id
        self.due_date = self._pattern_matching(
            due_date, "^\d{4}-\d{2}-\d{2}$", "due_date"
        )
        self.contract_id = contract_id



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee import Employee
from .page_info_without_cursor_new import PageInfoWithoutCursorNew


@JsonMap({})
class PeopleContainer(BaseModel):
    """PeopleContainer

    :param data: data
    :type data: List[Employee]
    :param page: page, defaults to None
    :type page: PageInfoWithoutCursorNew, optional
    """

    def __init__(self, data: List[Employee], page: PageInfoWithoutCursorNew = None):
        self.data = self._define_list(data, Employee)
        if page is not None:
            self.page = self._define_object(page, PageInfoWithoutCursorNew)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class MilestoneToCreateFormWithFile(BaseModel):
    """MilestoneToCreateFormWithFile

    :param amount: Amount to be paid for the milestone.
    :type amount: str
    :param title: Title of milestone.
    :type title: str
    :param description: Description of milestone.
    :type description: str
    :param file: Upload the file you want to attach to this entry., defaults to None
    :type file: str, optional
    """

    def __init__(self, amount: str, title: str, description: str, file: str = None):
        self.amount = amount
        self.title = title
        self.description = description
        if file is not None:
            self.file = file



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_created import InvoiceAdjustmentCreated


@JsonMap({})
class InvoiceAdjustmentCreatedContainer(BaseModel):
    """InvoiceAdjustmentCreatedContainer

    :param data: Details of invoice adjustment created.
    :type data: InvoiceAdjustmentCreated
    """

    def __init__(self, data: InvoiceAdjustmentCreated):
        self.data = self._define_object(data, InvoiceAdjustmentCreated)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_to_create import InvoiceAdjustmentToCreate


@JsonMap({})
class InvoiceAdjustmentToCreateContainer(BaseModel):
    """InvoiceAdjustmentToCreateContainer

    :param data: Details of invoice adjustment to create.
    :type data: InvoiceAdjustmentToCreate
    """

    def __init__(self, data: InvoiceAdjustmentToCreate):
        self.data = self._define_object(data, InvoiceAdjustmentToCreate)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GpEmployeeAddressUpdated(BaseModel):
    """GpEmployeeAddressUpdated

    :param country: Country code., defaults to None
    :type country: str, optional
    :param state: State name., defaults to None
    :type state: str, optional
    :param city: City name., defaults to None
    :type city: str, optional
    :param street: Street name., defaults to None
    :type street: str, optional
    :param zip: Zip code., defaults to None
    :type zip: str, optional
    :param updated_at: Long date-time format following ISO-8601, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        country: str = None,
        state: str = None,
        city: str = None,
        street: str = None,
        zip: str = None,
        updated_at: str = None,
    ):
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")
        if state is not None:
            self.state = state
        if city is not None:
            self.city = city
        if street is not None:
            self.street = street
        if zip is not None:
            self.zip = zip
        if updated_at is not None:
            self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .legal_entity_type import LegalEntityType


@JsonMap({"id_": "id", "type_": "type"})
class ClientOfContractLegalEntity5(BaseModel):
    """ClientOfContractLegalEntity5

    :param id_: id_
    :type id_: IdentifierValue
    :param name: name
    :type name: str
    :param email: User's email address.
    :type email: str
    :param type_: type_
    :type type_: LegalEntityType
    :param subtype: subtype
    :type subtype: str
    :param registration_number: registration_number
    :type registration_number: str
    :param vat_number: vat_number
    :type vat_number: str
    """

    def __init__(
        self,
        id_: IdentifierValue,
        name: str,
        email: str,
        type_: LegalEntityType,
        subtype: str,
        registration_number: str,
        vat_number: str,
    ):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.name = name
        self.email = email
        self.type_ = self._enum_matching(type_, LegalEntityType.list(), "type_")
        self.subtype = subtype
        self.registration_number = registration_number
        self.vat_number = vat_number



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .alternate_email_item import AlternateEmailItem


@JsonMap({"id_": "id"})
class WorkerOfBasicContract(BaseModel):
    """WorkerOfBasicContract

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param full_name: full_name
    :type full_name: str
    :param email: User's email address.
    :type email: str
    :param alternate_email: List of alternate email addresses., defaults to None
    :type alternate_email: List[AlternateEmailItem], optional
    :param image: image, defaults to None
    :type image: str, optional
    """

    def __init__(
        self,
        id_: str,
        full_name: str,
        email: str,
        alternate_email: List[AlternateEmailItem] = None,
        image: str = None,
    ):
        self.id_ = id_
        self.full_name = full_name
        self.email = email
        if alternate_email is not None:
            self.alternate_email = self._define_list(
                alternate_email, AlternateEmailItem
            )
        if image is not None:
            self.image = image



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class InvitationsOfBasicContract(BaseModel):
    """InvitationsOfBasicContract

    :param client_email: User's email address., defaults to None
    :type client_email: str, optional
    :param worker_email: User's email address., defaults to None
    :type worker_email: str, optional
    """

    def __init__(self, client_email: str = None, worker_email: str = None):
        if client_email is not None:
            self.client_email = client_email
        if worker_email is not None:
            self.worker_email = worker_email



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Benefits(BaseModel):
    """Benefits

    :param provider_id: The ID of a benefits provider to include in the calculation.
    :type provider_id: str
    :param plan_id: The ID of a benefits plan to include in the calculation., defaults to None
    :type plan_id: str, optional
    """

    def __init__(self, provider_id: str, plan_id: str = None):
        self.provider_id = provider_id
        if plan_id is not None:
            self.plan_id = plan_id


@JsonMap({})
class EorEmployeeCostCalculationRequestBody(BaseModel):
    """EorEmployeeCostCalculationRequestBody

    :param salary: The gross annual salary of the employee.
    :type salary: float
    :param country: The country of employment.
    :type country: str
    :param currency: The currency that the salary is paid in.
    :type currency: str
    :param benefits: Benefits which should be included in the calculation., defaults to None
    :type benefits: List[Benefits], optional
    """

    def __init__(
        self,
        salary: float,
        country: str,
        currency: str,
        benefits: List[Benefits] = None,
    ):
        self.salary = salary
        self.country = country
        self.currency = currency
        if benefits is not None:
            self.benefits = self._define_list(benefits, Benefits)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel
from .hris_direct_employee_details import HrisDirectEmployeeDetails
from .hris_team_information import HrisTeamInformation
from .hris_compensation import HrisCompensation
from .hris_job_information_title_id import HrisJobInformationTitleId
from .hris_job_information_title_name import HrisJobInformationTitleName
from .hris_contract_part_time import HrisContractPartTime
from .hris_contract_full_time import HrisContractFullTime


class HrisDirectEmployeeJobInformationGuard(OneOfBaseModel):
    class_list = {
        "HrisJobInformationTitleId": HrisJobInformationTitleId,
        "HrisJobInformationTitleName": HrisJobInformationTitleName,
    }


HrisDirectEmployeeJobInformation = Union[
    HrisJobInformationTitleId, HrisJobInformationTitleName
]


class HrisDirectEmployeeContractGuard(OneOfBaseModel):
    class_list = {
        "HrisContractPartTime": HrisContractPartTime,
        "HrisContractFullTime": HrisContractFullTime,
    }


HrisDirectEmployeeContract = Union[HrisContractPartTime, HrisContractFullTime]


@JsonMap({})
class HrisDirectEmployeeVacationInfo(BaseModel):
    """HrisDirectEmployeeVacationInfo

    :param vacation_accrual_start_date: vacation_accrual_start_date
    :type vacation_accrual_start_date: str
    :param vacation_yearly_policy: vacation_yearly_policy
    :type vacation_yearly_policy: float
    """

    def __init__(self, vacation_accrual_start_date: str, vacation_yearly_policy: float):
        self.vacation_accrual_start_date = vacation_accrual_start_date
        self.vacation_yearly_policy = vacation_yearly_policy


@JsonMap({})
class HrisDirectEmployee(BaseModel):
    """HrisDirectEmployee

    :param employee_details: employee_details
    :type employee_details: HrisDirectEmployeeDetails
    :param team_information: team_information
    :type team_information: HrisTeamInformation
    :param job_information: job_information
    :type job_information: HrisDirectEmployeeJobInformation
    :param compensation: compensation
    :type compensation: HrisCompensation
    :param contract: contract
    :type contract: HrisDirectEmployeeContract
    :param vacation_info: vacation_info, defaults to None
    :type vacation_info: HrisDirectEmployeeVacationInfo, optional
    """

    def __init__(
        self,
        employee_details: HrisDirectEmployeeDetails,
        team_information: HrisTeamInformation,
        job_information: HrisDirectEmployeeJobInformation,
        compensation: HrisCompensation,
        contract: HrisDirectEmployeeContract,
        vacation_info: HrisDirectEmployeeVacationInfo = None,
    ):
        self.employee_details = self._define_object(
            employee_details, HrisDirectEmployeeDetails
        )
        self.team_information = self._define_object(
            team_information, HrisTeamInformation
        )
        self.job_information = HrisDirectEmployeeJobInformationGuard.return_one_of(
            job_information
        )
        self.compensation = self._define_object(compensation, HrisCompensation)
        self.contract = HrisDirectEmployeeContractGuard.return_one_of(contract)
        if vacation_info is not None:
            self.vacation_info = self._define_object(
                vacation_info, HrisDirectEmployeeVacationInfo
            )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GenericResultDeletedData(BaseModel):
    """GenericResultDeletedData

    :param deleted: Confirms the deletion.
    :type deleted: bool
    """

    def __init__(self, deleted: bool):
        self.deleted = deleted


@JsonMap({})
class GenericResultDeleted(BaseModel):
    """GenericResultDeleted

    :param data: data
    :type data: GenericResultDeletedData
    """

    def __init__(self, data: GenericResultDeletedData):
        self.data = self._define_object(data, GenericResultDeletedData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import Union
from typing import List
from .base import OneOfBaseModel
from .legal_entity_type import LegalEntityType


class GetInvoiceListEntitiesGuard(OneOfBaseModel):
    class_list = {
        "List[LegalEntityType]": List[LegalEntityType],
        "LegalEntityTypeEnum": str,
    }


GetInvoiceListEntities = Union[List[LegalEntityType], str]



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class InvoiceDownloadObject(BaseModel):
    """InvoiceDownloadObject

    :param url: URL to requested invoice to download.
    :type url: str
    """

    def __init__(self, url: str):
        self.url = url



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class PeoplePayment(BaseModel):
    """PeoplePayment

    :param rate: The payment rate, defaults to None
    :type rate: float, optional
    :param scale: The payment scale (e.g., hourly, weekly, monthly, etc.), defaults to None
    :type scale: str, optional
    :param currency: The currency code (ISO 4217) for the payment, defaults to None
    :type currency: str, optional
    :param contract_name: The name of the contract associated with the payment, defaults to None
    :type contract_name: str, optional
    """

    def __init__(
        self,
        rate: float = None,
        scale: str = None,
        currency: str = None,
        contract_name: str = None,
    ):
        if rate is not None:
            self.rate = rate
        if scale is not None:
            self.scale = scale
        if currency is not None:
            self.currency = currency
        if contract_name is not None:
            self.contract_name = contract_name



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ValidationTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar MIN_LENGTH: "MIN_LENGTH"
    :vartype MIN_LENGTH: str
    :cvar MAX_LENGTH: "MAX_LENGTH"
    :vartype MAX_LENGTH: str
    :cvar REGEX: "REGEX"
    :vartype REGEX: str
    """

    MIN_LENGTH = "MIN_LENGTH"
    MAX_LENGTH = "MAX_LENGTH"
    REGEX = "REGEX"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ValidationTypeEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.timeoff_type_list_container import TimeoffTypeListContainer
from ..models.seniority_list_container import SeniorityListContainer
from ..models.job_title_list_container import JobTitleListContainer
from ..models.currency_list_container import CurrencyListContainer
from ..models.country_list_container import CountryListContainer


class LookupsService(BaseService):

    @cast_models
    def get_countries(self) -> CountryListContainer:
        """Retrieve a list of countries supported by Deel.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: CountryListContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/lookups/countries", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CountryListContainer._unmap(response)

    @cast_models
    def get_currencies(self) -> CurrencyListContainer:
        """Retrieve the list of currencies used by Deel.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: CurrencyListContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/lookups/currencies", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CurrencyListContainer._unmap(response)

    @cast_models
    def get_job_title_list(
        self, limit: float = None, after_cursor: str = None
    ) -> JobTitleListContainer:
        """Retrieve a list of pre-defined job titles in Deel platform.

        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param after_cursor: Return next page of results after given cursor, defaults to None
        :type after_cursor: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: JobTitleListContainer
        """

        Validator(float).is_optional().min(20).max(99).validate(limit)
        Validator(str).is_optional().validate(after_cursor)

        serialized_request = (
            Serializer(
                f"{self.base_url}/lookups/job-titles", self.get_default_headers()
            )
            .add_query("limit", limit)
            .add_query("after_cursor", after_cursor)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return JobTitleListContainer._unmap(response)

    @cast_models
    def get_seniority_list(self, limit: float = None) -> SeniorityListContainer:
        """Retrieve a list of pre-defined seniority level for roles in Deel platform.

        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: SeniorityListContainer
        """

        Validator(float).is_optional().min(20).max(99).validate(limit)

        serialized_request = (
            Serializer(
                f"{self.base_url}/lookups/seniorities", self.get_default_headers()
            )
            .add_query("limit", limit)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return SeniorityListContainer._unmap(response)

    @cast_models
    def get_timeoff_type_list(self) -> TimeoffTypeListContainer:
        """Retrieve a list of pre-defined time off types to register in Deel platform.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TimeoffTypeListContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/lookups/time-off-types", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TimeoffTypeListContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .final_payment_calculated import FinalPaymentCalculated


@JsonMap({})
class FinalPaymentCalculatedContainer(BaseModel):
    """FinalPaymentCalculatedContainer

    :param data: data
    :type data: FinalPaymentCalculated
    """

    def __init__(self, data: FinalPaymentCalculated):
        self.data = self._define_object(data, FinalPaymentCalculated)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Currency(BaseModel):
    """Currency

    :param code: code
    :type code: str
    :param name: name
    :type name: str
    """

    def __init__(self, code: str, name: str):
        self.code = self._pattern_matching(code, "^[A-Z]{3}$", "code")
        self.name = name



# This file was generated by liblab | https://liblab.com/

from typing import Union
from .base import OneOfBaseModel


class TimesheetIdItemsGuard(OneOfBaseModel):
    class_list = {"float": float, "str": str}


TimesheetIdItems = Union[float, str]



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.webhook_list_response import WebhookListResponse
from ..models.webhook_item_response import WebhookItemResponse
from ..models.webhook_event_type_list_response import WebhookEventTypeListResponse
from ..models.utils.cast_models import cast_models
from ..models.patch_webhook_request import PatchWebhookRequest
from ..models.create_webhook_request import CreateWebhookRequest


class WebhooksService(BaseService):

    @cast_models
    def get_all_webhooks(self) -> WebhookListResponse:
        """Retrieve a list of webhook subscriptions.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WebhookListResponse
        """

        serialized_request = (
            Serializer(f"{self.base_url}/webhooks", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WebhookListResponse._unmap(response)

    @cast_models
    def create_webhook(self, request_body: CreateWebhookRequest) -> WebhookItemResponse:
        """Create a new webhooks subscription.

        :param request_body: The request body.
        :type request_body: CreateWebhookRequest
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WebhookItemResponse
        """

        Validator(CreateWebhookRequest).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/webhooks", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return WebhookItemResponse._unmap(response)

    @cast_models
    def webhook_controller_get_by_id(self, id_: str) -> WebhookItemResponse:
        """Retrieve a single webhook subscription.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WebhookItemResponse
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(f"{self.base_url}/webhooks/{{id}}", self.get_default_headers())
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WebhookItemResponse._unmap(response)

    @cast_models
    def webhook_controller_edit_by_id(
        self, request_body: PatchWebhookRequest, id_: str
    ) -> WebhookItemResponse:
        """Edit a webhook subscription.

        :param request_body: The request body.
        :type request_body: PatchWebhookRequest
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WebhookItemResponse
        """

        Validator(PatchWebhookRequest).validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(f"{self.base_url}/webhooks/{{id}}", self.get_default_headers())
            .add_path("id", id_)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return WebhookItemResponse._unmap(response)

    @cast_models
    def webhook_controller_delete_by_id(self, id_: str):
        """Delete a webhook subscription.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(f"{self.base_url}/webhooks/{{id}}", self.get_default_headers())
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def get_all_webhook_event_types(self) -> WebhookEventTypeListResponse:
        """Retrieve a list of webhook event types.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WebhookEventTypeListResponse
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/webhooks/events/types", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WebhookEventTypeListResponse._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ContractWhoReportsEnum(Enum):
    """An enumeration representing different categories.

    :cvar BOTH: "both"
    :vartype BOTH: str
    :cvar CLIENT: "client"
    :vartype CLIENT: str
    :cvar CONTRACTOR: "contractor"
    :vartype CONTRACTOR: str
    """

    BOTH = "both"
    CLIENT = "client"
    CONTRACTOR = "contractor"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ContractWhoReportsEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class Msa(BaseModel):
    """Msa

    :param id_: id_
    :type id_: str
    :param title: title
    :type title: str
    """

    def __init__(self, id_: str, title: str):
        self.id_ = id_
        self.title = title


@JsonMap({"id_": "id"})
class AgreementClientLegalEntity(BaseModel):
    """AgreementClientLegalEntity

    :param id_: id_
    :type id_: IdentifierValue
    :param name: name
    :type name: str
    """

    def __init__(self, id_: IdentifierValue, name: str):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.name = name


@JsonMap({"id_": "id"})
class ProviderLegalEntity(BaseModel):
    """ProviderLegalEntity

    :param id_: id_
    :type id_: IdentifierValue
    :param name: name
    :type name: str
    """

    def __init__(self, id_: IdentifierValue, name: str):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.name = name


@JsonMap({"id_": "id"})
class Agreement(BaseModel):
    """Agreement

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param agreement_title: agreement_title
    :type agreement_title: str
    :param agreement_type: agreement_type
    :type agreement_type: str
    :param msa: msa
    :type msa: Msa
    :param client_legal_entity: client_legal_entity
    :type client_legal_entity: AgreementClientLegalEntity
    :param provider_legal_entity: provider_legal_entity
    :type provider_legal_entity: ProviderLegalEntity
    """

    def __init__(
        self,
        id_: str,
        agreement_title: str,
        agreement_type: str,
        msa: Msa,
        client_legal_entity: AgreementClientLegalEntity,
        provider_legal_entity: ProviderLegalEntity,
    ):
        self.id_ = id_
        self.agreement_title = agreement_title
        self.agreement_type = agreement_type
        self.msa = self._define_object(msa, Msa)
        self.client_legal_entity = self._define_object(
            client_legal_entity, AgreementClientLegalEntity
        )
        self.provider_legal_entity = self._define_object(
            provider_legal_entity, ProviderLegalEntity
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .payment import Payment
from .people_client_legal_entity import PeopleClientLegalEntity
from .monthly_payment import MonthlyPayment
from .employment import Employment


@JsonMap({"id_": "id"})
class InternalPeople(BaseModel):
    """People object representing a Deel

    :param id_: Unique identifier for the user, defaults to None
    :type id_: str, optional
    :param first_name: First name of the user, defaults to None
    :type first_name: str, optional
    :param last_name: Last name of the user, defaults to None
    :type last_name: str, optional
    :param full_name: Full name of the user, defaults to None
    :type full_name: str, optional
    :param email: Email address of the user, defaults to None
    :type email: str, optional
    :param work_email: Work email address of the user, defaults to None
    :type work_email: str, optional
    :param personal_email: Personal email address of the user, defaults to None
    :type personal_email: str, optional
    :param country_name: Name of the country where the user is located, defaults to None
    :type country_name: str, optional
    :param birth_date: Date of birth of the user in ISO format (yyyy-mm-dd), defaults to None
    :type birth_date: str, optional
    :param pic_url: URL of the user's profile picture, defaults to None
    :type pic_url: str, optional
    :param start_date: Date when the user started their current employment in ISO format (yyyy-mm-dd), defaults to None
    :type start_date: str, optional
    :param payments: List of payment objects representing the user's payment history, defaults to None
    :type payments: List[Payment], optional
    :param hourly_report_total: Total amount of hours worked by the user in ISO format (hh:mm:ss), defaults to None
    :type hourly_report_total: str, optional
    :param client_legal_entity: client_legal_entity, defaults to None
    :type client_legal_entity: PeopleClientLegalEntity, optional
    :param state: State where the user is located, defaults to None
    :type state: str, optional
    :param seniority: Seniority level of the user's role, defaults to None
    :type seniority: str, optional
    :param completion_date: Date when the user's current employment is expected to end in ISO format (yyyy-mm-ddThh:mm:ss.sssZ), defaults to None
    :type completion_date: str, optional
    :param monthly_payment: Monthly payment details for the user, defaults to None
    :type monthly_payment: MonthlyPayment, optional
    :param direct_manager: Name of the user's direct manager, defaults to None
    :type direct_manager: str, optional
    :param direct_reports: Names of the user's direct reports, defaults to None
    :type direct_reports: str, optional
    :param direct_reports_count: Number of direct reports that the user has, defaults to None
    :type direct_reports_count: int, optional
    :param employments: List of employment objects representing the user's employment history, defaults to None
    :type employments: List[Employment], optional
    :param hiring_status: Hiring status of the user, defaults to None
    :type hiring_status: str, optional
    :param hiring_type: Type of employment contract that the user has, defaults to None
    :type hiring_type: str, optional
    :param job_title: Title of the user's role, defaults to None
    :type job_title: str, optional
    :param country: ISO 3166-1 alpha-2 code of the country where the user is located, defaults to None
    :type country: str, optional
    :param team_id: Unique identifier for the team that the user belongs to, defaults to None
    :type team_id: int, optional
    :param team: Name of the team that the user belongs to, defaults to None
    :type team: str, optional
    :param timezone: Time zone of the user's location, defaults to None
    :type timezone: str, optional
    :param department: Name of the department that the user belongs to, defaults to None
    :type department: str, optional
    :param work_location: Name of the user's work location, defaults to None
    :type work_location: str, optional
    :param contract_types: List of contract types that the user has had in the past and/or currently has, defaults to None
    :type contract_types: List[str], optional
    :param has_deel_user: Indicates whether the user has a Deel account or not, defaults to None
    :type has_deel_user: bool, optional
    """

    def __init__(
        self,
        id_: str = None,
        first_name: str = None,
        last_name: str = None,
        full_name: str = None,
        email: str = None,
        work_email: str = None,
        personal_email: str = None,
        country_name: str = None,
        birth_date: str = None,
        pic_url: str = None,
        start_date: str = None,
        payments: List[Payment] = None,
        hourly_report_total: str = None,
        client_legal_entity: PeopleClientLegalEntity = None,
        state: str = None,
        seniority: str = None,
        completion_date: str = None,
        monthly_payment: MonthlyPayment = None,
        direct_manager: str = None,
        direct_reports: str = None,
        direct_reports_count: int = None,
        employments: List[Employment] = None,
        hiring_status: str = None,
        hiring_type: str = None,
        job_title: str = None,
        country: str = None,
        team_id: int = None,
        team: str = None,
        timezone: str = None,
        department: str = None,
        work_location: str = None,
        contract_types: List[str] = None,
        has_deel_user: bool = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if full_name is not None:
            self.full_name = full_name
        if email is not None:
            self.email = email
        if work_email is not None:
            self.work_email = work_email
        if personal_email is not None:
            self.personal_email = personal_email
        if country_name is not None:
            self.country_name = country_name
        if birth_date is not None:
            self.birth_date = birth_date
        if pic_url is not None:
            self.pic_url = pic_url
        if start_date is not None:
            self.start_date = start_date
        if payments is not None:
            self.payments = self._define_list(payments, Payment)
        if hourly_report_total is not None:
            self.hourly_report_total = hourly_report_total
        if client_legal_entity is not None:
            self.client_legal_entity = self._define_object(
                client_legal_entity, PeopleClientLegalEntity
            )
        if state is not None:
            self.state = state
        if seniority is not None:
            self.seniority = seniority
        if completion_date is not None:
            self.completion_date = completion_date
        if monthly_payment is not None:
            self.monthly_payment = self._define_object(monthly_payment, MonthlyPayment)
        if direct_manager is not None:
            self.direct_manager = direct_manager
        if direct_reports is not None:
            self.direct_reports = direct_reports
        if direct_reports_count is not None:
            self.direct_reports_count = direct_reports_count
        if employments is not None:
            self.employments = self._define_list(employments, Employment)
        if hiring_status is not None:
            self.hiring_status = hiring_status
        if hiring_type is not None:
            self.hiring_type = hiring_type
        if job_title is not None:
            self.job_title = job_title
        if country is not None:
            self.country = country
        if team_id is not None:
            self.team_id = team_id
        if team is not None:
            self.team = team
        if timezone is not None:
            self.timezone = timezone
        if department is not None:
            self.department = department
        if work_location is not None:
            self.work_location = work_location
        if contract_types is not None:
            self.contract_types = contract_types
        if has_deel_user is not None:
            self.has_deel_user = has_deel_user



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class EmployeeTimeoffsEntitlementsItem(BaseModel):
    """EmployeeTimeoffsEntitlementsItem

    :param year: year
    :type year: float
    :param tracking_period: tracking_period
    :type tracking_period: str
    :param vacation_allowed: vacation_allowed
    :type vacation_allowed: str
    :param vacation_rollover: vacation_rollover
    :type vacation_rollover: str
    :param vacation_expired: vacation_expired
    :type vacation_expired: str
    :param vacation_payout: vacation_payout
    :type vacation_payout: str
    :param vacation_requested: vacation_requested
    :type vacation_requested: str
    :param vacation_approved: vacation_approved
    :type vacation_approved: str
    :param vacation_used: vacation_used
    :type vacation_used: str
    :param vacation_total: vacation_total
    :type vacation_total: str
    :param sick_leave_requested: sick_leave_requested
    :type sick_leave_requested: str
    :param sick_leave_approved: sick_leave_approved
    :type sick_leave_approved: str
    :param sick_leave_used: sick_leave_used
    :type sick_leave_used: str
    :param sick_leave_total: sick_leave_total
    :type sick_leave_total: str
    :param sick_leave_allowed: sick_leave_allowed
    :type sick_leave_allowed: str
    :param other_leave_requested: other_leave_requested
    :type other_leave_requested: str
    :param other_leave_approved: other_leave_approved
    :type other_leave_approved: str
    :param other_leave_used: other_leave_used
    :type other_leave_used: str
    :param other_leave_total: other_leave_total
    :type other_leave_total: str
    """

    def __init__(
        self,
        year: float,
        tracking_period: str,
        vacation_allowed: str,
        vacation_rollover: str,
        vacation_expired: str,
        vacation_payout: str,
        vacation_requested: str,
        vacation_approved: str,
        vacation_used: str,
        vacation_total: str,
        sick_leave_requested: str,
        sick_leave_approved: str,
        sick_leave_used: str,
        sick_leave_total: str,
        sick_leave_allowed: str,
        other_leave_requested: str,
        other_leave_approved: str,
        other_leave_used: str,
        other_leave_total: str,
    ):
        self.year = year
        self.tracking_period = tracking_period
        self.vacation_allowed = vacation_allowed
        self.vacation_rollover = vacation_rollover
        self.vacation_expired = vacation_expired
        self.vacation_payout = vacation_payout
        self.vacation_requested = vacation_requested
        self.vacation_approved = vacation_approved
        self.vacation_used = vacation_used
        self.vacation_total = vacation_total
        self.sick_leave_requested = sick_leave_requested
        self.sick_leave_approved = sick_leave_approved
        self.sick_leave_used = sick_leave_used
        self.sick_leave_total = sick_leave_total
        self.sick_leave_allowed = sick_leave_allowed
        self.other_leave_requested = other_leave_requested
        self.other_leave_approved = other_leave_approved
        self.other_leave_used = other_leave_used
        self.other_leave_total = other_leave_total



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_client_timeoff_requests import EorClientTimeoffRequests


@JsonMap({"id_": "id"})
class EorTimeoffsEmployeeItem(BaseModel):
    """EorTimeoffsEmployeeItem

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param name: name
    :type name: str
    :param eor_contract_id: eor_contract_id
    :type eor_contract_id: float
    :param contract_id: contract_id
    :type contract_id: str
    :param vacations: List of employee vacation time offs.
    :type vacations: List[EorClientTimeoffRequests]
    :param sick_leaves: List of employee sick leave time offs.
    :type sick_leaves: List[EorClientTimeoffRequests]
    :param others: List of all other employee time offs.
    :type others: List[EorClientTimeoffRequests]
    :param time_zones: time_zones
    :type time_zones: str
    """

    def __init__(
        self,
        id_: str,
        name: str,
        eor_contract_id: float,
        contract_id: str,
        vacations: List[EorClientTimeoffRequests],
        sick_leaves: List[EorClientTimeoffRequests],
        others: List[EorClientTimeoffRequests],
        time_zones: str,
    ):
        self.id_ = id_
        self.name = name
        self.eor_contract_id = eor_contract_id
        self.contract_id = contract_id
        self.vacations = self._define_list(vacations, EorClientTimeoffRequests)
        self.sick_leaves = self._define_list(sick_leaves, EorClientTimeoffRequests)
        self.others = self._define_list(others, EorClientTimeoffRequests)
        self.time_zones = time_zones



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class PeopleTimeOffContainerType(Enum):
    """An enumeration representing different categories.

    :cvar VACATION: "VACATION"
    :vartype VACATION: str
    :cvar SICK_LEAVE: "SICK_LEAVE"
    :vartype SICK_LEAVE: str
    :cvar OTHER: "OTHER"
    :vartype OTHER: str
    """

    VACATION = "VACATION"
    SICK_LEAVE = "SICK_LEAVE"
    OTHER = "OTHER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PeopleTimeOffContainerType._member_map_.values())
        )


@JsonMap({"type_": "type"})
class PeopleTimeOffContainer(BaseModel):
    """PeopleTimeOffContainer

    :param type_: Time off type., defaults to None
    :type type_: PeopleTimeOffContainerType, optional
    :param start_date: Start date of employee's time off., defaults to None
    :type start_date: str, optional
    :param end_date: End date of employee's time off., defaults to None
    :type end_date: str, optional
    :param reason: Description for employee's time off., defaults to None
    :type reason: str, optional
    :param attachments: Attachments for employee's time off., defaults to None
    :type attachments: str, optional
    """

    def __init__(
        self,
        type_: PeopleTimeOffContainerType = None,
        start_date: str = None,
        end_date: str = None,
        reason: str = None,
        attachments: str = None,
    ):
        if type_ is not None:
            self.type_ = self._enum_matching(
                type_, PeopleTimeOffContainerType.list(), "type_"
            )
        if start_date is not None:
            self.start_date = start_date
        if end_date is not None:
            self.end_date = end_date
        if reason is not None:
            self.reason = reason
        if attachments is not None:
            self.attachments = attachments



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class ContractTerminationResult(BaseModel):
    """ContractTerminationResult

    :param id_: Deel Contract ID., defaults to None
    :type id_: str, optional
    :param completion_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type completion_date: str, optional
    """

    def __init__(self, id_: str = None, completion_date: str = None):
        if id_ is not None:
            self.id_ = id_
        if completion_date is not None:
            self.completion_date = self._pattern_matching(
                completion_date, "^\d{4}-\d{2}-\d{2}$", "completion_date"
            )



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.timesheet_to_update_container import TimesheetToUpdateContainer
from ..models.timesheet_to_create_container import TimesheetToCreateContainer
from ..models.timesheet_status_enum import TimesheetStatusEnum
from ..models.timesheet_reviews_to_create_container import (
    TimesheetReviewsToCreateContainer,
)
from ..models.timesheet_review_to_create_container import (
    TimesheetReviewToCreateContainer,
)
from ..models.timesheet_list_container import TimesheetListContainer
from ..models.timesheet_container import TimesheetContainer
from ..models.invoice_adjustment_created_container import (
    InvoiceAdjustmentCreatedContainer,
)
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.generic_result_created import GenericResultCreated
from ..models.contract_type_enum import ContractTypeEnum


class TimesheetsService(BaseService):

    @cast_models
    def get_timesheets_by_contract(
        self,
        contract_id: str,
        contract_types: List[ContractTypeEnum] = None,
        statuses: List[TimesheetStatusEnum] = None,
        reporter_id: str = None,
        date_from: str = None,
        date_to: str = None,
        limit: float = None,
        offset: float = None,
    ) -> TimesheetListContainer:
        """Retrieve a list of timesheets found for a contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param contract_types: Types of contracts to filter., defaults to None
        :type contract_types: List[ContractTypeEnum], optional
        :param statuses: Statuses of timesheets to filter., defaults to None
        :type statuses: List[TimesheetStatusEnum], optional
        :param reporter_id: ID of an existing profile; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type reporter_id: str, optional
        :param date_from: Filtered results will include records created on or after the provided date., defaults to None
        :type date_from: str, optional
        :param date_to: Filtered results will include records created before the provided date., defaults to None
        :type date_to: str, optional
        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row, defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TimesheetListContainer
        """

        Validator(str).validate(contract_id)
        Validator(ContractTypeEnum).is_array().is_optional().validate(contract_types)
        Validator(TimesheetStatusEnum).is_array().is_optional().validate(statuses)
        Validator(str).is_optional().validate(reporter_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_from)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_to)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/timesheets",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("contract_types", contract_types)
            .add_query("statuses", statuses)
            .add_query("reporter_id", reporter_id)
            .add_query("date_from", date_from, nullable=True)
            .add_query("date_to", date_to, nullable=True)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TimesheetListContainer._unmap(response)

    @cast_models
    def get_timesheets(
        self,
        contract_id: str = None,
        contract_types: List[ContractTypeEnum] = None,
        statuses: List[TimesheetStatusEnum] = None,
        reporter_id: str = None,
        date_from: str = None,
        date_to: str = None,
        limit: float = None,
        offset: float = None,
    ) -> TimesheetListContainer:
        """Retrieve a list of timesheets in your Deel account. You can filter the list by providing additional paramters e.g. contract_id, contract_type etc.

        :param contract_id: Id of a Deel contract., defaults to None
        :type contract_id: str, optional
        :param contract_types: Types of contracts to filter., defaults to None
        :type contract_types: List[ContractTypeEnum], optional
        :param statuses: Statuses of timesheets to filter., defaults to None
        :type statuses: List[TimesheetStatusEnum], optional
        :param reporter_id: ID of an existing profile; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type reporter_id: str, optional
        :param date_from: Filtered results will include records created on or after the provided date., defaults to None
        :type date_from: str, optional
        :param date_to: Filtered results will include records created before the provided date., defaults to None
        :type date_to: str, optional
        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row, defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TimesheetListContainer
        """

        Validator(str).is_optional().validate(contract_id)
        Validator(ContractTypeEnum).is_array().is_optional().validate(contract_types)
        Validator(TimesheetStatusEnum).is_array().is_optional().validate(statuses)
        Validator(str).is_optional().validate(reporter_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_from)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_to)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(f"{self.base_url}/timesheets", self.get_default_headers())
            .add_query("contract_id", contract_id)
            .add_query("contract_types", contract_types)
            .add_query("statuses", statuses)
            .add_query("reporter_id", reporter_id)
            .add_query("date_from", date_from, nullable=True)
            .add_query("date_to", date_to, nullable=True)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TimesheetListContainer._unmap(response)

    @cast_models
    def create_timesheet(
        self, request_body: TimesheetToCreateContainer
    ) -> InvoiceAdjustmentCreatedContainer:
        """Submit work for a contractor.

        :param request_body: The request body.
        :type request_body: TimesheetToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceAdjustmentCreatedContainer
        """

        Validator(TimesheetToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/timesheets", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return InvoiceAdjustmentCreatedContainer._unmap(response)

    @cast_models
    def get_timesheet_by_id(self, timesheet_id: str) -> TimesheetContainer:
        """Retrieve a single timesheet entry by Id.

        :param timesheet_id: ID of an existing timesheet
        :type timesheet_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TimesheetContainer
        """

        Validator(str).validate(timesheet_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/timesheets/{{timesheet_id}}",
                self.get_default_headers(),
            )
            .add_path("timesheet_id", timesheet_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TimesheetContainer._unmap(response)

    @cast_models
    def update_timesheet_by_id(
        self, request_body: TimesheetToUpdateContainer, timesheet_id: str
    ) -> GenericResultUpdated:
        """Update a single timesheet entry.

        :param request_body: The request body.
        :type request_body: TimesheetToUpdateContainer
        :param timesheet_id: ID of an existing timesheet
        :type timesheet_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(TimesheetToUpdateContainer).validate(request_body)
        Validator(str).validate(timesheet_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/timesheets/{{timesheet_id}}",
                self.get_default_headers(),
            )
            .add_path("timesheet_id", timesheet_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def delete_timesheet_by_id(
        self, timesheet_id: str, reason: str = None
    ) -> GenericResultDeleted:
        """Delete a single timesheet entry.

        :param timesheet_id: ID of an existing timesheet
        :type timesheet_id: str
        :param reason: Reason for deleting an existing timesheet, defaults to None
        :type reason: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(timesheet_id)
        Validator(str).is_optional().validate(reason)

        serialized_request = (
            Serializer(
                f"{self.base_url}/timesheets/{{timesheet_id}}",
                self.get_default_headers(),
            )
            .add_path("timesheet_id", timesheet_id)
            .add_query("reason", reason)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def create_timesheet_review(
        self, timesheet_id: str, request_body: TimesheetReviewToCreateContainer = None
    ) -> GenericResultCreated:
        """Review a timesheet to approve or decline submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: TimesheetReviewToCreateContainer, optional
        :param timesheet_id: ID of an existing timesheet
        :type timesheet_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(TimesheetReviewToCreateContainer).is_optional().validate(request_body)
        Validator(str).validate(timesheet_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/timesheets/{{timesheet_id}}/reviews",
                self.get_default_headers(),
            )
            .add_path("timesheet_id", timesheet_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def create_timesheet_reviews(
        self, request_body: TimesheetReviewsToCreateContainer = None
    ) -> GenericResultCreated:
        """Review a batch of timesheets to approve or reject submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: TimesheetReviewsToCreateContainer, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(TimesheetReviewsToCreateContainer).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url}/timesheets/many/reviews", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class WorkStatementCycleScaleEnum(Enum):
    """An enumeration representing different categories.

    :cvar WEEKLY: "weekly"
    :vartype WEEKLY: str
    :cvar MONTHLY: "monthly"
    :vartype MONTHLY: str
    :cvar BIWEEKLY: "biweekly"
    :vartype BIWEEKLY: str
    :cvar SEMIMONTHLY: "semimonthly"
    :vartype SEMIMONTHLY: str
    :cvar CALENDAR_MONTH: "calendar-month"
    :vartype CALENDAR_MONTH: str
    """

    WEEKLY = "weekly"
    MONTHLY = "monthly"
    BIWEEKLY = "biweekly"
    SEMIMONTHLY = "semimonthly"
    CALENDAR_MONTH = "calendar-month"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, WorkStatementCycleScaleEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ContractInvitationToCreate(BaseModel):
    """ContractInvitationToCreate

    :param email: Worker's email address.
    :type email: str
    :param message: Message for the worker.
    :type message: str
    """

    def __init__(self, email: str, message: str):
        self.email = email
        self.message = message



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee_timeoffs_item import EmployeeTimeoffsItem


@JsonMap({})
class EmployeeTimeoffsContainer(BaseModel):
    """EmployeeTimeoffsContainer

    :param data: List of employee time offs., defaults to None
    :type data: List[EmployeeTimeoffsItem], optional
    """

    def __init__(self, data: List[EmployeeTimeoffsItem] = None):
        if data is not None:
            self.data = self._define_list(data, EmployeeTimeoffsItem)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ContractExternalIdToPatch(BaseModel):
    """ContractExternalIdToPatch

    :param external_id: A unique identifier for the object provided by an external system.
    :type external_id: str
    """

    def __init__(self, external_id: str):
        self.external_id = external_id



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class GpContractSalaryScaleEnum(Enum):
    """An enumeration representing different categories.

    :cvar YEAR: "YEAR"
    :vartype YEAR: str
    :cvar MONTH: "MONTH"
    :vartype MONTH: str
    :cvar HOUR: "HOUR"
    :vartype HOUR: str
    """

    YEAR = "YEAR"
    MONTH = "MONTH"
    HOUR = "HOUR"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, GpContractSalaryScaleEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .file_attachment_info import FileAttachmentInfo


@JsonMap({})
class PgoTaskToCreate(BaseModel):
    """PgoTaskToCreate

    :param amount: Fixed rate for this task.
    :type amount: str
    :param date_submitted: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type date_submitted: str
    :param description: Task description.
    :type description: str
    :param attachment: This object is used for linking file attachments to your records., defaults to None
    :type attachment: FileAttachmentInfo, optional
    """

    def __init__(
        self,
        amount: str,
        date_submitted: str,
        description: str,
        attachment: FileAttachmentInfo = None,
    ):
        self.amount = amount
        self.date_submitted = date_submitted
        self.description = description
        if attachment is not None:
            self.attachment = self._define_object(attachment, FileAttachmentInfo)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel
from .validation_type_enum import ValidationTypeEnum


class ValueGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


Value = Union[str, float]


@JsonMap({"type_": "type"})
class ValidationType(BaseModel):
    """ValidationType

    :param type_: type_, defaults to None
    :type type_: ValidationTypeEnum, optional
    :param value: Validation value., defaults to None
    :type value: Value, optional
    """

    def __init__(self, type_: ValidationTypeEnum = None, value: Value = None):
        if type_ is not None:
            self.type_ = self._enum_matching(type_, ValidationTypeEnum.list(), "type_")
        if value is not None:
            self.value = ValueGuard.return_one_of(value)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .webhook_event_type_response import WebhookEventTypeResponse


@JsonMap({})
class WebhookEventTypeListResponse(BaseModel):
    """WebhookEventTypeListResponse

    :param data: data, defaults to None
    :type data: List[WebhookEventTypeResponse], optional
    """

    def __init__(self, data: List[WebhookEventTypeResponse] = None):
        if data is not None:
            self.data = self._define_list(data, WebhookEventTypeResponse)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


class CreateWebhookRequestStatus(Enum):
    """An enumeration representing different categories.

    :cvar ENABLED: "enabled"
    :vartype ENABLED: str
    :cvar DISABLED: "disabled"
    :vartype DISABLED: str
    """

    ENABLED = "enabled"
    DISABLED = "disabled"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, CreateWebhookRequestStatus._member_map_.values())
        )


@JsonMap({})
class CreateWebhookRequest(BaseModel):
    """CreateWebhookRequest

    :param name: Webhook subscription name.
    :type name: str
    :param description: Webhook subscription description.
    :type description: str
    :param status: Status of webhook.
    :type status: CreateWebhookRequestStatus
    :param url: Endpoint to receive webhook.
    :type url: str
    :param signing_key: The webhook's signing key, used to generate webhook signatures.
    :type signing_key: str
    :param api_version: Deel API version. Currently Deel accepts v1 or v2 version.
    :type api_version: str
    :param events: The list of events to enable for this subscription.
    :type events: List[str]
    """

    def __init__(
        self,
        name: str,
        description: str,
        status: CreateWebhookRequestStatus,
        url: str,
        signing_key: str,
        api_version: str,
        events: List[str],
    ):
        self.name = name
        self.description = description
        self.status = self._enum_matching(
            status, CreateWebhookRequestStatus.list(), "status"
        )
        self.url = url
        self.signing_key = signing_key
        self.api_version = api_version
        self.events = events



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class HrisDirectEmployeeDetails(BaseModel):
    """HrisDirectEmployeeDetails

    :param first_name: first_name
    :type first_name: str
    :param last_name: last_name
    :type last_name: str
    :param email: email
    :type email: str
    :param work_email: work_email
    :type work_email: str
    :param nationality: nationality
    :type nationality: str
    :param country: country
    :type country: str
    :param state: state, defaults to None
    :type state: str, optional
    """

    def __init__(
        self,
        first_name: str,
        last_name: str,
        email: str,
        work_email: str,
        nationality: str,
        country: str,
        state: str = None,
    ):
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.work_email = work_email
        self.nationality = nationality
        self.country = country
        if state is not None:
            self.state = state



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .payment_method_enum import PaymentMethodEnum


@JsonMap({"type_": "type"})
class PaymentMethod(BaseModel):
    """PaymentMethod

    :param type_: All the supported payment types, defaults to None
    :type type_: PaymentMethodEnum, optional
    """

    def __init__(self, type_: PaymentMethodEnum = None):
        if type_ is not None:
            self.type_ = self._enum_matching(type_, PaymentMethodEnum.list(), "type_")



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_type_enum_for_estimate import ContractTypeEnumForEstimate
from .work_statement_cycle_scale_enum import WorkStatementCycleScaleEnum
from .work_statement_cycle_end_type_enum import WorkStatementCycleEndTypeEnum
from .work_statement_payment_due_type_enum import WorkStatementPaymentDueTypeEnum
from .week_days_enum import WeekDaysEnum


class CompensationDetailsCalculationType(Enum):
    """An enumeration representing different categories.

    :cvar CUSTOM_AMOUNT: "CUSTOM_AMOUNT"
    :vartype CUSTOM_AMOUNT: str
    :cvar WORK_DAYS: "WORK_DAYS"
    :vartype WORK_DAYS: str
    :cvar CALENDAR_DAYS: "CALENDAR_DAYS"
    :vartype CALENDAR_DAYS: str
    :cvar FULL_AMOUNT: "FULL_AMOUNT"
    :vartype FULL_AMOUNT: str
    """

    CUSTOM_AMOUNT = "CUSTOM_AMOUNT"
    WORK_DAYS = "WORK_DAYS"
    CALENDAR_DAYS = "CALENDAR_DAYS"
    FULL_AMOUNT = "FULL_AMOUNT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                CompensationDetailsCalculationType._member_map_.values(),
            )
        )


@JsonMap({})
class EstimateFirstPaymentCompensationDetails(BaseModel):
    """EstimateFirstPaymentCompensationDetails

    :param amount: Amount to be paid. Must be a positive number., defaults to None
    :type amount: float, optional
    :param currency_code: currency_code, defaults to None
    :type currency_code: str, optional
    :param scale: Scale of the invoice cycle., defaults to None
    :type scale: WorkStatementCycleScaleEnum, optional
    :param cycle_end: Date invoice cycle ends., defaults to None
    :type cycle_end: float, optional
    :param cycle_end_type: Cycle end can be weekly 1-7 (Monday = 1) or monthly 1-31., defaults to None
    :type cycle_end_type: WorkStatementCycleEndTypeEnum, optional
    :param payment_due_type: Type of payment due. If the payment is due on the last day of month, enter 'WITHIN_MONTH'., defaults to None
    :type payment_due_type: WorkStatementPaymentDueTypeEnum, optional
    :param payment_due_days: payment_due_days, defaults to None
    :type payment_due_days: float, optional
    :param calculation_type: Either works days or calendar days, defaults to None
    :type calculation_type: CompensationDetailsCalculationType, optional
    :param work_week_start: Days of the week., defaults to None
    :type work_week_start: WeekDaysEnum, optional
    :param work_week_end: Days of the week., defaults to None
    :type work_week_end: WeekDaysEnum, optional
    """

    def __init__(
        self,
        amount: float = None,
        currency_code: str = None,
        scale: WorkStatementCycleScaleEnum = None,
        cycle_end: float = None,
        cycle_end_type: WorkStatementCycleEndTypeEnum = None,
        payment_due_type: WorkStatementPaymentDueTypeEnum = None,
        payment_due_days: float = None,
        calculation_type: CompensationDetailsCalculationType = None,
        work_week_start: WeekDaysEnum = None,
        work_week_end: WeekDaysEnum = None,
    ):
        if amount is not None:
            self.amount = amount
        if currency_code is not None:
            self.currency_code = self._pattern_matching(
                currency_code, "^[A-Z]{3}$", "currency_code"
            )
        if scale is not None:
            self.scale = self._enum_matching(
                scale, WorkStatementCycleScaleEnum.list(), "scale"
            )
        if cycle_end is not None:
            self.cycle_end = cycle_end
        if cycle_end_type is not None:
            self.cycle_end_type = self._enum_matching(
                cycle_end_type, WorkStatementCycleEndTypeEnum.list(), "cycle_end_type"
            )
        if payment_due_type is not None:
            self.payment_due_type = self._enum_matching(
                payment_due_type,
                WorkStatementPaymentDueTypeEnum.list(),
                "payment_due_type",
            )
        if payment_due_days is not None:
            self.payment_due_days = payment_due_days
        if calculation_type is not None:
            self.calculation_type = self._enum_matching(
                calculation_type,
                CompensationDetailsCalculationType.list(),
                "calculation_type",
            )
        if work_week_start is not None:
            self.work_week_start = self._enum_matching(
                work_week_start, WeekDaysEnum.list(), "work_week_start"
            )
        if work_week_end is not None:
            self.work_week_end = self._enum_matching(
                work_week_end, WeekDaysEnum.list(), "work_week_end"
            )


@JsonMap({"type_": "type"})
class EstimateFirstPayment(BaseModel):
    """Details of pro rata payment to create.

    :param type_: Type of a contract
    :type type_: ContractTypeEnumForEstimate
    :param country_code: Country code.
    :type country_code: str
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param compensation_details: compensation_details
    :type compensation_details: EstimateFirstPaymentCompensationDetails
    """

    def __init__(
        self,
        type_: ContractTypeEnumForEstimate,
        country_code: str,
        start_date: str,
        compensation_details: EstimateFirstPaymentCompensationDetails,
    ):
        self.type_ = self._enum_matching(
            type_, ContractTypeEnumForEstimate.list(), "type_"
        )
        self.country_code = self._pattern_matching(
            country_code, "^[A-Z]{2}$", "country_code"
        )
        self.start_date = start_date
        self.compensation_details = self._define_object(
            compensation_details, EstimateFirstPaymentCompensationDetails
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_type_to_create_enum import InvoiceAdjustmentTypeToCreateEnum


@JsonMap({"type_": "type", "payment_cycle_id": "paymentCycleId"})
class InvoiceAdjustmentToCreateWithFile(BaseModel):
    """InvoiceAdjustmentToCreateWithFile

    :param contract_id: Id of a Deel contract.
    :type contract_id: str
    :param date_submitted: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type date_submitted: str
    :param type_: Type of invoice adjustment.
    :type type_: InvoiceAdjustmentTypeToCreateEnum
    :param amount: Amount to be paid. Must be a positive number.
    :type amount: float
    :param description: Description of the adjustment.
    :type description: str
    :param payment_cycle_id: ID of an existing active payment cycle - required if type is "vat", defaults to None
    :type payment_cycle_id: float, optional
    :param file: Upload the file you want to attach to this entry., defaults to None
    :type file: str, optional
    """

    def __init__(
        self,
        contract_id: str,
        date_submitted: str,
        type_: InvoiceAdjustmentTypeToCreateEnum,
        amount: float,
        description: str,
        payment_cycle_id: float = None,
        file: str = None,
    ):
        self.contract_id = contract_id
        self.date_submitted = date_submitted
        self.type_ = self._enum_matching(
            type_, InvoiceAdjustmentTypeToCreateEnum.list(), "type_"
        )
        self.amount = amount
        self.description = description
        if payment_cycle_id is not None:
            self.payment_cycle_id = payment_cycle_id
        if file is not None:
            self.file = file



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .bank_account_value_allowed import BankAccountValueAllowed
from .validation_type import ValidationType


@JsonMap({"type_": "type"})
class BankAccountGuide(BaseModel):
    """BankAccountGuide

    :param key: The key of the field.
    :type key: str
    :param label: Label for this field., defaults to None
    :type label: str, optional
    :param required: Whether the field is required or not.
    :type required: bool
    :param values_allowed: values_allowed, defaults to None
    :type values_allowed: List[BankAccountValueAllowed], optional
    :param validations: validations, defaults to None
    :type validations: List[ValidationType], optional
    :param type_: Type of the field, defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        key: str,
        required: bool,
        label: str = None,
        values_allowed: List[BankAccountValueAllowed] = None,
        validations: List[ValidationType] = None,
        type_: str = None,
    ):
        self.key = key
        if label is not None:
            self.label = label
        self.required = required
        if values_allowed is not None:
            self.values_allowed = self._define_list(
                values_allowed, BankAccountValueAllowed
            )
        if validations is not None:
            self.validations = self._define_list(validations, ValidationType)
        if type_ is not None:
            self.type_ = type_



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.equity_stakeholders_container import EquityStakeholdersContainer


class CartaService(BaseService):

    @cast_models
    def get_equity_stakeholders(
        self, limit: float = None, cursor: str = None
    ) -> EquityStakeholdersContainer:
        """Retrieve all stakeholders for Carta.

        :param limit: Return a page of results with given number of records; NOTE technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param cursor: Return next page of results after given cursor., defaults to None
        :type cursor: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EquityStakeholdersContainer
        """

        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(str).is_optional().validate(cursor)

        serialized_request = (
            Serializer(
                f"{self.base_url}/equity/stakeholders", self.get_default_headers()
            )
            .add_query("limit", limit)
            .add_query("cursor", cursor)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EquityStakeholdersContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class PublicToken(BaseModel):
    """PublicToken

    :param token: Public token.
    :type token: str
    """

    def __init__(self, token: str):
        self.token = token



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "pic_url": "picUrl"})
class PaymentWorker(BaseModel):
    """PaymentWorker

    :param id_: Worker's unique id., defaults to None
    :type id_: float, optional
    :param name: Worker's full name.
    :type name: str
    :param pic_url: URL to worker's Deel avatar.
    :type pic_url: str
    :param contract_id: The worker's Deel contract Id., defaults to None
    :type contract_id: str, optional
    """

    def __init__(
        self, name: str, pic_url: str, id_: float = None, contract_id: str = None
    ):
        if id_ is not None:
            self.id_ = id_
        self.name = name
        self.pic_url = pic_url
        if contract_id is not None:
            self.contract_id = contract_id



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"type_": "type"})
class Email(BaseModel):
    """Email

    :param type_: The type of email., defaults to None
    :type type_: str, optional
    :param value: The email address., defaults to None
    :type value: str, optional
    """

    def __init__(self, type_: str = None, value: str = None):
        if type_ is not None:
            self.type_ = type_
        if value is not None:
            self.value = value



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class HealthInsuranceProviderAttachments(BaseModel):
    """HealthInsuranceProviderAttachments

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param label: label, defaults to None
    :type label: str, optional
    """

    def __init__(self, id_: IdentifierValue = None, label: str = None):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if label is not None:
            self.label = label


@JsonMap({"id_": "id"})
class Plans(BaseModel):
    """Plans

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param name: Insurance plan name., defaults to None
    :type name: str, optional
    :param price: Insurance price., defaults to None
    :type price: str, optional
    :param currency: Insurance payment currency., defaults to None
    :type currency: str, optional
    :param is_enabled: is_enabled, defaults to None
    :type is_enabled: bool, optional
    """

    def __init__(
        self,
        id_: IdentifierValue = None,
        name: str = None,
        price: str = None,
        currency: str = None,
        is_enabled: bool = None,
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if name is not None:
            self.name = name
        if price is not None:
            self.price = price
        if currency is not None:
            self.currency = currency
        if is_enabled is not None:
            self.is_enabled = is_enabled


@JsonMap({"id_": "id", "type_": "type"})
class HealthInsuranceProvider(BaseModel):
    """Health insurance provider.

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param name: Name of insurance provider., defaults to None
    :type name: str, optional
    :param is_unisure: is_unisure, defaults to None
    :type is_unisure: bool, optional
    :param home_page_url: Insurance provider's home page url., defaults to None
    :type home_page_url: str, optional
    :param currency: Payment currency., defaults to None
    :type currency: str, optional
    :param type_: Insurance type., defaults to None
    :type type_: str, optional
    :param ending_rule: Insurance cycle ending rule., defaults to None
    :type ending_rule: str, optional
    :param days_to_cancel: Days to cancel the insurance., defaults to None
    :type days_to_cancel: str, optional
    :param pricing_info_link: URL to price info., defaults to None
    :type pricing_info_link: str, optional
    :param fixed_price: URL to price info., defaults to None
    :type fixed_price: bool, optional
    :param attachments: File attachments., defaults to None
    :type attachments: List[HealthInsuranceProviderAttachments], optional
    :param client_info_banner: Health insurance info banner., defaults to None
    :type client_info_banner: str, optional
    :param is_available_for_new_quotes: is_available_for_new_quotes, defaults to None
    :type is_available_for_new_quotes: bool, optional
    :param plans: plans, defaults to None
    :type plans: List[Plans], optional
    """

    def __init__(
        self,
        id_: IdentifierValue = None,
        name: str = None,
        is_unisure: bool = None,
        home_page_url: str = None,
        currency: str = None,
        type_: str = None,
        ending_rule: str = None,
        days_to_cancel: str = None,
        pricing_info_link: str = None,
        fixed_price: bool = None,
        attachments: List[HealthInsuranceProviderAttachments] = None,
        client_info_banner: str = None,
        is_available_for_new_quotes: bool = None,
        plans: List[Plans] = None,
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if name is not None:
            self.name = name
        if is_unisure is not None:
            self.is_unisure = is_unisure
        if home_page_url is not None:
            self.home_page_url = home_page_url
        if currency is not None:
            self.currency = currency
        if type_ is not None:
            self.type_ = type_
        if ending_rule is not None:
            self.ending_rule = ending_rule
        if days_to_cancel is not None:
            self.days_to_cancel = days_to_cancel
        if pricing_info_link is not None:
            self.pricing_info_link = pricing_info_link
        if fixed_price is not None:
            self.fixed_price = fixed_price
        if attachments is not None:
            self.attachments = self._define_list(
                attachments, HealthInsuranceProviderAttachments
            )
        if client_info_banner is not None:
            self.client_info_banner = client_info_banner
        if is_available_for_new_quotes is not None:
            self.is_available_for_new_quotes = is_available_for_new_quotes
        if plans is not None:
            self.plans = self._define_list(plans, Plans)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class SignaturesOfContract(BaseModel):
    """SignaturesOfContract

    :param client_signature: normally name of client is used as signature
    :type client_signature: str
    :param client_signed_at: Long date-time format following ISO-8601
    :type client_signed_at: str
    :param worker_signature: Signatures of client representative.
    :type worker_signature: str
    :param worker_signed_at: Long date-time format following ISO-8601
    :type worker_signed_at: str
    :param signed_at: Long date-time format following ISO-8601
    :type signed_at: str
    """

    def __init__(
        self,
        client_signature: str,
        client_signed_at: str,
        worker_signature: str,
        worker_signed_at: str,
        signed_at: str,
    ):
        self.client_signature = client_signature
        self.client_signed_at = client_signed_at
        self.worker_signature = worker_signature
        self.worker_signed_at = worker_signed_at
        self.signed_at = signed_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class Departments(BaseModel):
    """Departments

    :param id_: id_
    :type id_: float
    :param name: name
    :type name: str
    :param parent: parent, defaults to None
    :type parent: str, optional
    """

    def __init__(self, id_: float, name: str, parent: str = None):
        self.id_ = id_
        self.name = name
        if parent is not None:
            self.parent = parent



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .payment_method import PaymentMethod
from .payment_status_enum import PaymentStatusEnum
from .payment_worker import PaymentWorker


@JsonMap({"id_": "id"})
class Payment(BaseModel):
    """Payment

    :param id_: Unique identifier of the payment.
    :type id_: float
    :param payment_method: payment_method
    :type payment_method: PaymentMethod
    :param status: status of payment
    :type status: PaymentStatusEnum
    :param payment_currency: Currency code.
    :type payment_currency: str
    :param label: label
    :type label: str
    :param paid_at: Long date-time format following ISO-8601
    :type paid_at: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param total: Total paid.
    :type total: str
    :param workers: workers
    :type workers: List[PaymentWorker]
    """

    def __init__(
        self,
        id_: float,
        payment_method: PaymentMethod,
        status: PaymentStatusEnum,
        payment_currency: str,
        label: str,
        paid_at: str,
        created_at: str,
        total: str,
        workers: List[PaymentWorker],
    ):
        self.id_ = id_
        self.payment_method = self._define_object(payment_method, PaymentMethod)
        self.status = self._enum_matching(status, PaymentStatusEnum.list(), "status")
        self.payment_currency = self._pattern_matching(
            payment_currency, "^[A-Z]{3}$", "payment_currency"
        )
        self.label = label
        self.paid_at = paid_at
        self.created_at = created_at
        self.total = total
        self.workers = self._define_list(workers, PaymentWorker)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class PensionEorContractToCreate(BaseModel):
    """Some countries required a pension benefit.

    :param id_: Pension provider id. You can see available pension providers in the country guide endpoint., defaults to None
    :type id_: str, optional
    :param contribution: Enter the value of pension contribution. You should send this value only if the contribution object is available in the country guide endpoint and the value should be there between minimum and maximum values indicated., defaults to None
    :type contribution: str, optional
    """

    def __init__(self, id_: str = None, contribution: str = None):
        if id_ is not None:
            self.id_ = id_
        if contribution is not None:
            self.contribution = contribution



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class SeniorityRequired(BaseModel):
    """Seniority describes level of expertise at a job e.g. junior

    :param id_: id_
    :type id_: IdentifierValue
    :param name: Name of seniority level e.g. Mid (Individual Contributor Level 2)
    :type name: str
    :param level: Level of seniority level e.g. 2
    :type level: float
    """

    def __init__(self, id_: IdentifierValue, name: str, level: float):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.name = name
        self.level = level



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .candidate_to_patch import CandidateToPatch


@JsonMap({})
class CandidateToPatchContainer(BaseModel):
    """CandidateToPatchContainer

    :param data: Details of candidate to update
    :type data: CandidateToPatch
    """

    def __init__(self, data: CandidateToPatch):
        self.data = self._define_object(data, CandidateToPatch)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class OffCyclePaymentToCreate(BaseModel):
    """Details of invoice adjustment to create.

    :param date_submitted: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type date_submitted: str
    :param amount: Amount to be paid. Must be a positive number.
    :type amount: float
    :param description: Description of off-cycle payment., defaults to None
    :type description: str, optional
    """

    def __init__(self, date_submitted: str, amount: float, description: str = None):
        self.date_submitted = date_submitted
        self.amount = amount
        if description is not None:
            self.description = description



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .basic_organization import BasicOrganization


@JsonMap({})
class OrganizationListContainer(BaseModel):
    """OrganizationListContainer

    :param data: data
    :type data: List[BasicOrganization]
    """

    def __init__(self, data: List[BasicOrganization]):
        self.data = self._define_list(data, BasicOrganization)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_field import EorField
from .salary_frequency_scale_enum import SalaryFrequencyScaleEnum
from .benefit_status_enum import BenefitStatusEnum
from .health_insurance_provider import HealthInsuranceProvider
from .pension_provider import PensionProvider


@JsonMap({"most_common": "mostCommon"})
class Holiday(BaseModel):
    """Range of the number of holidays.

    :param min: Minimum number of holidays required for a legally compliant contract., defaults to None
    :type min: str, optional
    :param max: Maximum number of holidays required for a legally compliant contract., defaults to None
    :type max: str, optional
    :param most_common: Most common number of holidays, defaults to None
    :type most_common: str, optional
    """

    def __init__(self, min: str = None, max: str = None, most_common: str = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max
        if most_common is not None:
            self.most_common = most_common


class PartTimeHolidayType(Enum):
    """An enumeration representing different categories.

    :cvar PRORATED: "PRORATED"
    :vartype PRORATED: str
    :cvar STANDARD: "STANDARD"
    :vartype STANDARD: str
    :cvar SPECIFIC: "SPECIFIC"
    :vartype SPECIFIC: str
    """

    PRORATED = "PRORATED"
    STANDARD = "STANDARD"
    SPECIFIC = "SPECIFIC"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PartTimeHolidayType._member_map_.values()))


@JsonMap({"type_": "type"})
class PartTimeHoliday(BaseModel):
    """Holiday type and range for part-time contracts.

    :param type_: type_, defaults to None
    :type type_: PartTimeHolidayType, optional
    :param min: Minimum number of holidays required for a legally compliant contract., defaults to None
    :type min: str, optional
    :param max: Maximum number of holidays required for a legally compliant contract., defaults to None
    :type max: str, optional
    """

    def __init__(
        self, type_: PartTimeHolidayType = None, min: str = None, max: str = None
    ):
        if type_ is not None:
            self.type_ = self._enum_matching(type_, PartTimeHolidayType.list(), "type_")
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class SickDays(BaseModel):
    """SickDays

    :param min: Minimum number of sick days required for a legally compliant contract., defaults to None
    :type min: str, optional
    :param max: Maximum number of sick days allowed for a legally compliant contract., defaults to None
    :type max: str, optional
    """

    def __init__(self, min: str = None, max: str = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class Salary(BaseModel):
    """Salary range.

    :param min: Minimum wage for a legally compliant contract., defaults to None
    :type min: str, optional
    :param max: Maximum wage allowed for a legally compliant contract., defaults to None
    :type max: str, optional
    :param frequency: Scale of the salary frequency., defaults to None
    :type frequency: SalaryFrequencyScaleEnum, optional
    """

    def __init__(
        self,
        min: str = None,
        max: str = None,
        frequency: SalaryFrequencyScaleEnum = None,
    ):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max
        if frequency is not None:
            self.frequency = self._enum_matching(
                frequency, SalaryFrequencyScaleEnum.list(), "frequency"
            )


@JsonMap({})
class Probation(BaseModel):
    """Probation period range.

    :param min: Minimum probation days required for a legally compliant contract., defaults to None
    :type min: str, optional
    :param max: Maximum probation days allowed for a legally compliant contract., defaults to None
    :type max: float, optional
    """

    def __init__(self, min: str = None, max: float = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class PartTimeProbation(BaseModel):
    """Probation period range for part-time contracts.

    :param min: Minimum probation days required for a legally compliant contract., defaults to None
    :type min: float, optional
    :param max: Maximum probation days allowed for a legally compliant contract., defaults to None
    :type max: float, optional
    """

    def __init__(self, min: float = None, max: float = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class Days(BaseModel):
    """Days

    :param min: min, defaults to None
    :type min: str, optional
    :param max: max, defaults to None
    :type max: str, optional
    """

    def __init__(self, min: str = None, max: str = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class Hours(BaseModel):
    """Hours

    :param min: min, defaults to None
    :type min: str, optional
    :param max: max, defaults to None
    :type max: str, optional
    """

    def __init__(self, min: str = None, max: str = None):
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max


@JsonMap({})
class WorkSchedule(BaseModel):
    """Employee's Work schedule in days and hours.

    :param days: days, defaults to None
    :type days: Days, optional
    :param hours: hours, defaults to None
    :type hours: Hours, optional
    """

    def __init__(self, days: Days = None, hours: Hours = None):
        if days is not None:
            self.days = self._define_object(days, Days)
        if hours is not None:
            self.hours = self._define_object(hours, Hours)


class DefiniteContractType(Enum):
    """An enumeration representing different categories.

    :cvar ALLOWED_WITHOUT_LIMITATION: "ALLOWED_WITHOUT_LIMITATION"
    :vartype ALLOWED_WITHOUT_LIMITATION: str
    :cvar ALLOWED_WITH_MAXIMUM_LIMITATION: "ALLOWED_WITH_MAXIMUM_LIMITATION"
    :vartype ALLOWED_WITH_MAXIMUM_LIMITATION: str
    :cvar NOT_ALLOWED: "NOT_ALLOWED"
    :vartype NOT_ALLOWED: str
    """

    ALLOWED_WITHOUT_LIMITATION = "ALLOWED_WITHOUT_LIMITATION"
    ALLOWED_WITH_MAXIMUM_LIMITATION = "ALLOWED_WITH_MAXIMUM_LIMITATION"
    NOT_ALLOWED = "NOT_ALLOWED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, DefiniteContractType._member_map_.values()))


@JsonMap({"type_": "type"})
class DefiniteContract(BaseModel):
    """Conditions to make definitive contract.

    :param type_: type_, defaults to None
    :type type_: DefiniteContractType, optional
    :param maximum_limitation: maximum_limitation, defaults to None
    :type maximum_limitation: float, optional
    """

    def __init__(
        self, type_: DefiniteContractType = None, maximum_limitation: float = None
    ):
        if type_ is not None:
            self.type_ = self._enum_matching(
                type_, DefiniteContractType.list(), "type_"
            )
        if maximum_limitation is not None:
            self.maximum_limitation = maximum_limitation


@JsonMap({})
class HealthInsurance(BaseModel):
    """HealthInsurance

    :param status: status, defaults to None
    :type status: BenefitStatusEnum, optional
    :param providers: providers, defaults to None
    :type providers: List[HealthInsuranceProvider], optional
    """

    def __init__(
        self,
        status: BenefitStatusEnum = None,
        providers: List[HealthInsuranceProvider] = None,
    ):
        if status is not None:
            self.status = self._enum_matching(
                status, BenefitStatusEnum.list(), "status"
            )
        if providers is not None:
            self.providers = self._define_list(providers, HealthInsuranceProvider)


@JsonMap({})
class Pension(BaseModel):
    """If it is there, you must specify the pension benefit when creating an EOR contract.

    :param status: status, defaults to None
    :type status: BenefitStatusEnum, optional
    :param providers: providers, defaults to None
    :type providers: List[PensionProvider], optional
    """

    def __init__(
        self, status: BenefitStatusEnum = None, providers: List[PensionProvider] = None
    ):
        if status is not None:
            self.status = self._enum_matching(
                status, BenefitStatusEnum.list(), "status"
            )
        if providers is not None:
            self.providers = self._define_list(providers, PensionProvider)


@JsonMap({})
class EorCountryValidations(BaseModel):
    """EorCountryValidations

    :param holiday: Range of the number of holidays., defaults to None
    :type holiday: Holiday, optional
    :param part_time_holiday: Holiday type and range for part-time contracts., defaults to None
    :type part_time_holiday: PartTimeHoliday, optional
    :param sick_days: sick_days, defaults to None
    :type sick_days: SickDays, optional
    :param salary: Salary range., defaults to None
    :type salary: Salary, optional
    :param probation: Probation period range., defaults to None
    :type probation: Probation, optional
    :param part_time_probation: Probation period range for part-time contracts., defaults to None
    :type part_time_probation: PartTimeProbation, optional
    :param work_schedule: Employee's Work schedule in days and hours., defaults to None
    :type work_schedule: WorkSchedule, optional
    :param insurance_fee: Insurance fee., defaults to None
    :type insurance_fee: str, optional
    :param currency: Country currency., defaults to None
    :type currency: str, optional
    :param hiring_guide_country_name: Country name., defaults to None
    :type hiring_guide_country_name: str, optional
    :param start_date_buffer: Calculate employee's minimum start date using the start date buffer. Today's date + (number of business days x start_date_buffer) = Minimum start date., defaults to None
    :type start_date_buffer: float, optional
    :param definite_contract: Conditions to make definitive contract., defaults to None
    :type definite_contract: DefiniteContract, optional
    :param adjustments_information_box: Country specific notes and information., defaults to None
    :type adjustments_information_box: str, optional
    :param health_insurance: health_insurance, defaults to None
    :type health_insurance: HealthInsurance, optional
    :param pension: If it is there, you must specify the pension benefit when creating an EOR contract., defaults to None
    :type pension: Pension, optional
    :param mandatory_fields: mandatory_fields, defaults to None
    :type mandatory_fields: List[EorField], optional
    """

    def __init__(
        self,
        holiday: Holiday = None,
        part_time_holiday: PartTimeHoliday = None,
        sick_days: SickDays = None,
        salary: Salary = None,
        probation: Probation = None,
        part_time_probation: PartTimeProbation = None,
        work_schedule: WorkSchedule = None,
        insurance_fee: str = None,
        currency: str = None,
        hiring_guide_country_name: str = None,
        start_date_buffer: float = None,
        definite_contract: DefiniteContract = None,
        adjustments_information_box: str = None,
        health_insurance: HealthInsurance = None,
        pension: Pension = None,
        mandatory_fields: List[EorField] = None,
    ):
        if holiday is not None:
            self.holiday = self._define_object(holiday, Holiday)
        if part_time_holiday is not None:
            self.part_time_holiday = self._define_object(
                part_time_holiday, PartTimeHoliday
            )
        if sick_days is not None:
            self.sick_days = self._define_object(sick_days, SickDays)
        if salary is not None:
            self.salary = self._define_object(salary, Salary)
        if probation is not None:
            self.probation = self._define_object(probation, Probation)
        if part_time_probation is not None:
            self.part_time_probation = self._define_object(
                part_time_probation, PartTimeProbation
            )
        if work_schedule is not None:
            self.work_schedule = self._define_object(work_schedule, WorkSchedule)
        if insurance_fee is not None:
            self.insurance_fee = insurance_fee
        if currency is not None:
            self.currency = currency
        if hiring_guide_country_name is not None:
            self.hiring_guide_country_name = hiring_guide_country_name
        if start_date_buffer is not None:
            self.start_date_buffer = start_date_buffer
        if definite_contract is not None:
            self.definite_contract = self._define_object(
                definite_contract, DefiniteContract
            )
        if adjustments_information_box is not None:
            self.adjustments_information_box = adjustments_information_box
        if health_insurance is not None:
            self.health_insurance = self._define_object(
                health_insurance, HealthInsurance
            )
        if pension is not None:
            self.pension = self._define_object(pension, Pension)
        if mandatory_fields is not None:
            self.mandatory_fields = self._define_list(mandatory_fields, EorField)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .address import Address
from .email import Email
from .people_client_legal_entity import PeopleClientLegalEntity
from .employment_detail import EmploymentDetail
from .people_custom_field import PeopleCustomField


@JsonMap({"id_": "id"})
class DataDirectManager(BaseModel):
    """The direct manager of the employee.

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param last_name: last_name, defaults to None
    :type last_name: str, optional
    :param first_name: first_name, defaults to None
    :type first_name: str, optional
    :param work_email: work_email, defaults to None
    :type work_email: str, optional
    :param worker_id: worker_id, defaults to None
    :type worker_id: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        last_name: str = None,
        first_name: str = None,
        work_email: str = None,
        worker_id: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if last_name is not None:
            self.last_name = last_name
        if first_name is not None:
            self.first_name = first_name
        if work_email is not None:
            self.work_email = work_email
        if worker_id is not None:
            self.worker_id = worker_id


@JsonMap({"id_": "id"})
class DataDirectReports(BaseModel):
    """DataDirectReports

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param last_name: last_name, defaults to None
    :type last_name: str, optional
    :param first_name: first_name, defaults to None
    :type first_name: str, optional
    :param work_email: work_email, defaults to None
    :type work_email: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        last_name: str = None,
        first_name: str = None,
        work_email: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if last_name is not None:
            self.last_name = last_name
        if first_name is not None:
            self.first_name = first_name
        if work_email is not None:
            self.work_email = work_email


@JsonMap({"id_": "id"})
class DataDepartment(BaseModel):
    """The department of the employee.

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param parent: parent, defaults to None
    :type parent: str, optional
    """

    def __init__(self, id_: str = None, name: str = None, parent: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if parent is not None:
            self.parent = parent


@JsonMap({"id_": "id"})
class PeopleByIdContainerData(BaseModel):
    """PeopleByIdContainerData

    :param id_: Unique identifier for the employee., defaults to None
    :type id_: str, optional
    :param created_at: The creation timestamp of the employee record., defaults to None
    :type created_at: str, optional
    :param first_name: The first name of the employee., defaults to None
    :type first_name: str, optional
    :param last_name: The last name of the employee., defaults to None
    :type last_name: str, optional
    :param full_name: The full name of the employee., defaults to None
    :type full_name: str, optional
    :param addresses: addresses, defaults to None
    :type addresses: List[Address], optional
    :param emails: An array of the employee's email addresses., defaults to None
    :type emails: List[Email], optional
    :param birth_date: The birth date of the employee., defaults to None
    :type birth_date: str, optional
    :param start_date: The date the employee started working., defaults to None
    :type start_date: str, optional
    :param nationality: The nationality of the employee., defaults to None
    :type nationality: str, optional
    :param client_legal_entity: client_legal_entity, defaults to None
    :type client_legal_entity: PeopleClientLegalEntity, optional
    :param state: The state of the employee., defaults to None
    :type state: str, optional
    :param seniority: The seniority level of the employee., defaults to None
    :type seniority: str, optional
    :param completion_date: The completion date of the task or project., defaults to None
    :type completion_date: str, optional
    :param direct_manager: The direct manager of the employee., defaults to None
    :type direct_manager: DataDirectManager, optional
    :param direct_reports: The direct reports to the employee., defaults to None
    :type direct_reports: List[DataDirectReports], optional
    :param direct_reports_count: The count of direct reports to the employee., defaults to None
    :type direct_reports_count: int, optional
    :param employments: employments, defaults to None
    :type employments: List[EmploymentDetail], optional
    :param hiring_status: The current hiring status of the employee., defaults to None
    :type hiring_status: str, optional
    :param new_hiring_status: The new hiring status of the employee., defaults to None
    :type new_hiring_status: str, optional
    :param hiring_type: The type of hiring of the employee., defaults to None
    :type hiring_type: str, optional
    :param job_title: The job title of the employee., defaults to None
    :type job_title: str, optional
    :param country: The country of the employee., defaults to None
    :type country: str, optional
    :param timezone: The timezone of the employee., defaults to None
    :type timezone: str, optional
    :param department: The department of the employee., defaults to None
    :type department: DataDepartment, optional
    :param work_location: The work location of the employee., defaults to None
    :type work_location: str, optional
    :param updated_at: Long date-time format following ISO-8601, defaults to None
    :type updated_at: str, optional
    :param custom_fields: custom_fields, defaults to None
    :type custom_fields: List[PeopleCustomField], optional
    """

    def __init__(
        self,
        id_: str = None,
        created_at: str = None,
        first_name: str = None,
        last_name: str = None,
        full_name: str = None,
        addresses: List[Address] = None,
        emails: List[Email] = None,
        birth_date: str = None,
        start_date: str = None,
        nationality: str = None,
        client_legal_entity: PeopleClientLegalEntity = None,
        state: str = None,
        seniority: str = None,
        completion_date: str = None,
        direct_manager: DataDirectManager = None,
        direct_reports: List[DataDirectReports] = None,
        direct_reports_count: int = None,
        employments: List[EmploymentDetail] = None,
        hiring_status: str = None,
        new_hiring_status: str = None,
        hiring_type: str = None,
        job_title: str = None,
        country: str = None,
        timezone: str = None,
        department: DataDepartment = None,
        work_location: str = None,
        updated_at: str = None,
        custom_fields: List[PeopleCustomField] = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if created_at is not None:
            self.created_at = created_at
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if full_name is not None:
            self.full_name = full_name
        if addresses is not None:
            self.addresses = self._define_list(addresses, Address)
        if emails is not None:
            self.emails = self._define_list(emails, Email)
        if birth_date is not None:
            self.birth_date = birth_date
        if start_date is not None:
            self.start_date = start_date
        if nationality is not None:
            self.nationality = nationality
        if client_legal_entity is not None:
            self.client_legal_entity = self._define_object(
                client_legal_entity, PeopleClientLegalEntity
            )
        if state is not None:
            self.state = state
        if seniority is not None:
            self.seniority = seniority
        if completion_date is not None:
            self.completion_date = completion_date
        if direct_manager is not None:
            self.direct_manager = self._define_object(direct_manager, DataDirectManager)
        if direct_reports is not None:
            self.direct_reports = self._define_list(direct_reports, DataDirectReports)
        if direct_reports_count is not None:
            self.direct_reports_count = direct_reports_count
        if employments is not None:
            self.employments = self._define_list(employments, EmploymentDetail)
        if hiring_status is not None:
            self.hiring_status = hiring_status
        if new_hiring_status is not None:
            self.new_hiring_status = new_hiring_status
        if hiring_type is not None:
            self.hiring_type = hiring_type
        if job_title is not None:
            self.job_title = job_title
        if country is not None:
            self.country = country
        if timezone is not None:
            self.timezone = timezone
        if department is not None:
            self.department = self._define_object(department, DataDepartment)
        if work_location is not None:
            self.work_location = work_location
        if updated_at is not None:
            self.updated_at = updated_at
        if custom_fields is not None:
            self.custom_fields = self._define_list(custom_fields, PeopleCustomField)


@JsonMap({})
class PeopleByIdContainer(BaseModel):
    """PeopleByIdContainer

    :param data: data
    :type data: PeopleByIdContainerData
    """

    def __init__(self, data: PeopleByIdContainerData):
        self.data = self._define_object(data, PeopleByIdContainerData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


class EmployeeTimeoffsItemType(Enum):
    """An enumeration representing different categories.

    :cvar VACATION: "VACATION"
    :vartype VACATION: str
    :cvar SICK_LEAVE: "SICK_LEAVE"
    :vartype SICK_LEAVE: str
    :cvar OTHER: "OTHER"
    :vartype OTHER: str
    """

    VACATION = "VACATION"
    SICK_LEAVE = "SICK_LEAVE"
    OTHER = "OTHER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EmployeeTimeoffsItemType._member_map_.values())
        )


@JsonMap({})
class Requester(BaseModel):
    """Requester

    :param name: Time off requester name., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class Reviewer(BaseModel):
    """Reviewer

    :param name: Time off reviewer name., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class EmployeeTimeoffsItemAttachments(BaseModel):
    """EmployeeTimeoffsItemAttachments

    :param attachment_id: Unique identifier of an attachment., defaults to None
    :type attachment_id: str, optional
    :param label: Attachment label., defaults to None
    :type label: str, optional
    :param file_extension: Attachment file extension., defaults to None
    :type file_extension: str, optional
    """

    def __init__(
        self, attachment_id: str = None, label: str = None, file_extension: str = None
    ):
        if attachment_id is not None:
            self.attachment_id = attachment_id
        if label is not None:
            self.label = label
        if file_extension is not None:
            self.file_extension = file_extension


@JsonMap({"type_": "type"})
class EmployeeTimeoffsItem(BaseModel):
    """EmployeeTimeoffsItem

    :param time_off_id: time_off_id
    :type time_off_id: IdentifierValue
    :param time_off_type_id: Unique identifier of this resource.
    :type time_off_type_id: str
    :param policy_id: Unique identifier of this resource.
    :type policy_id: str
    :param policy_type_id: Unique identifier of this resource.
    :type policy_type_id: str
    :param policy_name: Time off policy name.
    :type policy_name: str
    :param policy_type_name: Time off policy type name.
    :type policy_type_name: str
    :param reason: Reason for time off.
    :type reason: str
    :param type_: Time off type.
    :type type_: EmployeeTimeoffsItemType
    :param requested_at: Time off requested date.
    :type requested_at: str
    :param reviewed_at: Time off reviewed date.
    :type reviewed_at: str
    :param with_multiple_dates: Time off with multiple dates.
    :type with_multiple_dates: bool
    :param created_at: Time off created date.
    :type created_at: str
    :param updated_at: Time off updated date.
    :type updated_at: str
    :param denial_reason: Time off denial reason.
    :type denial_reason: str
    :param requester: requester
    :type requester: Requester
    :param reviewer: reviewer
    :type reviewer: Reviewer
    :param days_used_start_year: Days used in the start year.
    :type days_used_start_year: str
    :param days_used_end_year: Days used in the end year.
    :type days_used_end_year: str
    :param days_used: Days used.
    :type days_used: str
    :param start_date: Time off start date.
    :type start_date: str
    :param end_date: Time off end date.
    :type end_date: str
    :param is_start_date_half_day: Is start date half day.
    :type is_start_date_half_day: bool
    :param is_end_date_half_day: Is end date half day.
    :type is_end_date_half_day: bool
    :param attachments: attachments
    :type attachments: List[EmployeeTimeoffsItemAttachments]
    :param change_request: Time off change request.
    :type change_request: str
    """

    def __init__(
        self,
        time_off_id: IdentifierValue,
        time_off_type_id: str,
        policy_id: str,
        policy_type_id: str,
        policy_name: str,
        policy_type_name: str,
        reason: str,
        type_: EmployeeTimeoffsItemType,
        requested_at: str,
        reviewed_at: str,
        with_multiple_dates: bool,
        created_at: str,
        updated_at: str,
        denial_reason: str,
        requester: Requester,
        reviewer: Reviewer,
        days_used_start_year: str,
        days_used_end_year: str,
        days_used: str,
        start_date: str,
        end_date: str,
        is_start_date_half_day: bool,
        is_end_date_half_day: bool,
        attachments: List[EmployeeTimeoffsItemAttachments],
        change_request: str,
    ):
        self.time_off_id = IdentifierValueGuard.return_one_of(time_off_id)
        self.time_off_type_id = time_off_type_id
        self.policy_id = policy_id
        self.policy_type_id = policy_type_id
        self.policy_name = policy_name
        self.policy_type_name = policy_type_name
        self.reason = reason
        self.type_ = self._enum_matching(
            type_, EmployeeTimeoffsItemType.list(), "type_"
        )
        self.requested_at = requested_at
        self.reviewed_at = reviewed_at
        self.with_multiple_dates = with_multiple_dates
        self.created_at = created_at
        self.updated_at = updated_at
        self.denial_reason = denial_reason
        self.requester = self._define_object(requester, Requester)
        self.reviewer = self._define_object(reviewer, Reviewer)
        self.days_used_start_year = days_used_start_year
        self.days_used_end_year = days_used_end_year
        self.days_used = days_used
        self.start_date = start_date
        self.end_date = end_date
        self.is_start_date_half_day = is_start_date_half_day
        self.is_end_date_half_day = is_end_date_half_day
        self.attachments = self._define_list(
            attachments, EmployeeTimeoffsItemAttachments
        )
        self.change_request = change_request



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .timeoffs_type_enum import TimeoffsTypeEnum
from .timeoffs_status_enum import TimeoffsStatusEnum
from .timeoffs_profile import TimeoffsProfile
from .timeoffs_attachments_item import TimeoffsAttachmentsItem


@JsonMap({"type_": "type"})
class EorTimeoffsBaseItem(BaseModel):
    """EorTimeoffsBaseItem

    :param reason: Reason for requesting time off., defaults to None
    :type reason: str, optional
    :param type_: type_
    :type type_: TimeoffsTypeEnum
    :param requested_at: Long date-time format following ISO-8601
    :type requested_at: str
    :param reviewed_at: Long date-time format following ISO-8601, defaults to None
    :type reviewed_at: str, optional
    :param denial_reason: Reason why time off was denied., defaults to None
    :type denial_reason: str, optional
    :param has_multiple_dates: Indicates if multiple time off days are requested., defaults to None
    :type has_multiple_dates: bool, optional
    :param status: status
    :type status: TimeoffsStatusEnum
    :param other_timeoff_name: Custom title for the time off. Only specify if type is "OTHER"., defaults to None
    :type other_timeoff_name: str, optional
    :param requester: requester, defaults to None
    :type requester: TimeoffsProfile, optional
    :param reviewer: reviewer, defaults to None
    :type reviewer: TimeoffsProfile, optional
    :param days_used_start_year: Days used at the start year., defaults to None
    :type days_used_start_year: str, optional
    :param days_used_end_year: Days used at the end year., defaults to None
    :type days_used_end_year: str, optional
    :param total_days_used: Total number of time off days used., defaults to None
    :type total_days_used: str, optional
    :param start_date_is_half_day: Indicates if the first day of time off is a half day., defaults to None
    :type start_date_is_half_day: bool, optional
    :param end_date_is_half_day: Indicates if the last day of time off is a half day., defaults to None
    :type end_date_is_half_day: bool, optional
    :param single_date: Long date-time format following ISO-8601, defaults to None
    :type single_date: str, optional
    :param date_is_half_day: If true, only a single date was specified., defaults to None
    :type date_is_half_day: bool, optional
    :param attachments: A list of attachments. 2022-09-08 FE only allows for "SICK LEAVE", defaults to None
    :type attachments: List[TimeoffsAttachmentsItem], optional
    """

    def __init__(
        self,
        type_: TimeoffsTypeEnum,
        requested_at: str,
        status: TimeoffsStatusEnum,
        reason: str = None,
        reviewed_at: str = None,
        denial_reason: str = None,
        has_multiple_dates: bool = None,
        other_timeoff_name: str = None,
        requester: TimeoffsProfile = None,
        reviewer: TimeoffsProfile = None,
        days_used_start_year: str = None,
        days_used_end_year: str = None,
        total_days_used: str = None,
        start_date_is_half_day: bool = None,
        end_date_is_half_day: bool = None,
        single_date: str = None,
        date_is_half_day: bool = None,
        attachments: List[TimeoffsAttachmentsItem] = None,
    ):
        if reason is not None:
            self.reason = reason
        self.type_ = self._enum_matching(type_, TimeoffsTypeEnum.list(), "type_")
        self.requested_at = requested_at
        if reviewed_at is not None:
            self.reviewed_at = reviewed_at
        if denial_reason is not None:
            self.denial_reason = denial_reason
        if has_multiple_dates is not None:
            self.has_multiple_dates = has_multiple_dates
        self.status = self._enum_matching(status, TimeoffsStatusEnum.list(), "status")
        if other_timeoff_name is not None:
            self.other_timeoff_name = other_timeoff_name
        if requester is not None:
            self.requester = self._define_object(requester, TimeoffsProfile)
        if reviewer is not None:
            self.reviewer = self._define_object(reviewer, TimeoffsProfile)
        if days_used_start_year is not None:
            self.days_used_start_year = days_used_start_year
        if days_used_end_year is not None:
            self.days_used_end_year = days_used_end_year
        if total_days_used is not None:
            self.total_days_used = total_days_used
        if start_date_is_half_day is not None:
            self.start_date_is_half_day = start_date_is_half_day
        if end_date_is_half_day is not None:
            self.end_date_is_half_day = end_date_is_half_day
        if single_date is not None:
            self.single_date = single_date
        if date_is_half_day is not None:
            self.date_is_half_day = date_is_half_day
        if attachments is not None:
            self.attachments = self._define_list(attachments, TimeoffsAttachmentsItem)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class PeopleSortByEnum(Enum):
    """An enumeration representing different categories.

    :cvar ID: "id"
    :vartype ID: str
    :cvar FIRST_NAME: "first_name"
    :vartype FIRST_NAME: str
    :cvar LAST_NAME: "last_name"
    :vartype LAST_NAME: str
    :cvar FULL_NAME: "full_name"
    :vartype FULL_NAME: str
    :cvar EMAIL: "email"
    :vartype EMAIL: str
    :cvar COUNTRY: "country"
    :vartype COUNTRY: str
    :cvar BIRTH_DATE: "birth_date"
    :vartype BIRTH_DATE: str
    :cvar HIRING_TYPE: "hiring_type"
    :vartype HIRING_TYPE: str
    :cvar START_DATE: "start_date"
    :vartype START_DATE: str
    :cvar TEAM: "team"
    :vartype TEAM: str
    :cvar JOB_TITLE: "job_title"
    :vartype JOB_TITLE: str
    :cvar HIRING_STATUS: "hiring_status"
    :vartype HIRING_STATUS: str
    :cvar COMPLETION_DATE: "completion_date"
    :vartype COMPLETION_DATE: str
    :cvar DIRECT_MANAGER: "direct_manager"
    :vartype DIRECT_MANAGER: str
    :cvar DIRECT_REPORTS_COUNT: "direct_reports_count"
    :vartype DIRECT_REPORTS_COUNT: str
    """

    ID = "id"
    FIRST_NAME = "first_name"
    LAST_NAME = "last_name"
    FULL_NAME = "full_name"
    EMAIL = "email"
    COUNTRY = "country"
    BIRTH_DATE = "birth_date"
    HIRING_TYPE = "hiring_type"
    START_DATE = "start_date"
    TEAM = "team"
    JOB_TITLE = "job_title"
    HIRING_STATUS = "hiring_status"
    COMPLETION_DATE = "completion_date"
    DIRECT_MANAGER = "direct_manager"
    DIRECT_REPORTS_COUNT = "direct_reports_count"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PeopleSortByEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_created import GenericResultCreated
from ..models.candidate_to_patch_container import CandidateToPatchContainer
from ..models.candidate_to_create_container import CandidateToCreateContainer


class CandidatesService(BaseService):

    @cast_models
    def add_candidate(
        self, request_body: CandidateToCreateContainer
    ) -> GenericResultCreated:
        """Add a candidate to Deel.

        :param request_body: The request body.
        :type request_body: CandidateToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(CandidateToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/candidates", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def patch_candidate(
        self, request_body: CandidateToPatchContainer, candidate_id: str
    ) -> GenericResultUpdated:
        """Update an existed candidate.

        :param request_body: The request body.
        :type request_body: CandidateToPatchContainer
        :param candidate_id: Candidate id.
        :type candidate_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(CandidateToPatchContainer).validate(request_body)
        Validator(str).validate(candidate_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/candidates/{{candidate_id}}",
                self.get_default_headers(),
            )
            .add_path("candidate_id", candidate_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class MaritalStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar SINGLE: "Single"
    :vartype SINGLE: str
    :cvar MARRIED: "Married"
    :vartype MARRIED: str
    :cvar WIDOWED: "Widowed"
    :vartype WIDOWED: str
    :cvar DIVORCED: "Divorced"
    :vartype DIVORCED: str
    :cvar SEPARATED: "Separated"
    :vartype SEPARATED: str
    :cvar REGISTERED_PARTNERSHIP: "Registered Partnership"
    :vartype REGISTERED_PARTNERSHIP: str
    :cvar COMMON_LAW: "Common Law"
    :vartype COMMON_LAW: str
    """

    SINGLE = "Single"
    MARRIED = "Married"
    WIDOWED = "Widowed"
    DIVORCED = "Divorced"
    SEPARATED = "Separated"
    REGISTERED_PARTNERSHIP = "Registered Partnership"
    COMMON_LAW = "Common Law"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, MaritalStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"type_": "type", "id_": "id"})
class EmployeeTaxDocumentsList(BaseModel):
    """EmployeeTaxDocumentsList

    :param year: year, defaults to None
    :type year: float, optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param url: url, defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        year: float = None,
        type_: str = None,
        country: str = None,
        id_: str = None,
        url: str = None,
    ):
        if year is not None:
            self.year = year
        if type_ is not None:
            self.type_ = type_
        if country is not None:
            self.country = country
        if id_ is not None:
            self.id_ = id_
        if url is not None:
            self.url = url
